@@ -12,6 +12,9 @@
  * 2 of the License, or (at your option) any later version.
  * ----------------------------------------------------------------------------*/
 
+#include <stdarg.h>
+
+#define E_OK		0
 #define E_PERM		1
 #define E_PRIVATE	2
 #define E_NOEXIST	3
@@ -39,8 +42,14 @@
 #define E_INVALID	25
 #define E_CLDFAIL	26
 #define E_OLDPORT	27
+#define E_OPTEXISTS	28
+#define E_NODIR		29
+
+enum pbberrtype { PBB_ERR, PBB_WARN, PBB_INFO };
 
 /* prototypes */
-void print_error(char *msg, ...);
+void print_error (char *msg, ...);
+void print_msg (enum pbberrtype type, char *msg, ...);
+void print_message (enum pbberrtype type, char *msg, va_list args);
 
 #endif    /* INCLUDE_PBBERRNO_H */
