@@ -50,7 +50,7 @@ struct pbbmessage {
 
 /* public prototypes */
 int ipc_init (char *name, int mode, int reg);
-int ipc_exit ();
+void ipc_exit ();
 int ipc_send (int mp, int action, struct tagitem *taglist);
 int ipc_receive (void *buffer, size_t bufferlen);
 
