@@ -39,7 +39,6 @@
 #define TAG_AUTORESCAN			MOD_SYSTEM | 11
 #define TAG_TIMEFORCMD			MOD_SYSTEM | 12
 #define TAG_SYSINFO				MOD_SYSTEM | 13
-#define TAG_CFGNAME				MOD_SYSTEM | FLG_PRIVATE | FLG_STRING | 14
 #define TAG_REINIT				MOD_SYSTEM | 15
 #define TAG_SYSINFOSLAVE		MOD_SYSTEM | FLG_PRIVATE | 16
 
@@ -54,7 +53,6 @@
 #define TAG_PREPAREFORSLEEP		MOD_LAPTOP | FLG_PRIVATE | 8
 #define TAG_WAKEUPFROMSLEEP		MOD_LAPTOP | FLG_PRIVATE | 9
 #define TAG_TIMECHANGED			MOD_LAPTOP | FLG_PRIVATE | 10
-#define TAG_HW_BACKLIGHT		MOD_LAPTOP | FLG_PRIVATE | 11
 
 #define MOD_PMAC				3 << 16
 #define TAG_PMUDEVICE			MOD_PMAC | FLG_STRING | 1
@@ -65,7 +63,6 @@
 #define TAG_TPMODEDOWNMOD		MOD_PMAC | 6
 #define TAG_TPMODE				MOD_PMAC | 7
 #define TAG_KBDMODE				MOD_PMAC | 8
-//#define TAG_REPLACEPMUD			MOD_PMAC | 9   /* depreciated */
 #define TAG_IDENTITY			MOD_PMAC | FLG_STRING | 10
 #define TAG_POWERSOURCE			MOD_PMAC | 11
 #define TAG_TIMEREMAINING		MOD_PMAC | 12
@@ -76,6 +73,7 @@
 #define TAG_BATCYCLE			MOD_PMAC | 17
 #define TAG_BATTERYPRESENT		MOD_PMAC | 18
 #define TAG_NOTAPTYPING			MOD_PMAC | 19
+#define TAG_AMBIENTLIGHTMAX		MOD_PMAC | 20
 
 /* Pri2 Tags */
 #define MOD_DISPLAY				4 << 16
@@ -92,19 +90,17 @@
 #define TAG_LCDAUTOADJUST		MOD_DISPLAY | 12
 #define TAG_BRIGHTNESS			TAG_LCDBRIGHTNESS   /* depreciated */
 #define TAG_LCDBRIGHTNESS		MOD_DISPLAY | 5
-//#define TAG_LCDBRIGHTNESSMAX	MOD_DISPLAY | 7     /* deprecated */
 #define TAG_LCDFADINGSPEED		MOD_DISPLAY | 10
-#define TAG_LCDTHRESHOLD		MOD_DISPLAY | 24
-#define TAG_LCDAUTOADJMINBAT	MOD_DISPLAY | 25
-#define TAG_LCDAUTOADJMAXBAT	MOD_DISPLAY | 26
-#define TAG_LCDAUTOADJMINAC		MOD_DISPLAY | 27
-#define TAG_LCDAUTOADJMAXAC		MOD_DISPLAY | 28
+#define TAG_LCDTHRESHOLD		MOD_DISPLAY | 24    /* deprecated */
+#define TAG_LCDAUTOADJMINBAT	MOD_DISPLAY | 25    /* deprecated */
+#define TAG_LCDAUTOADJMAXBAT	MOD_DISPLAY | 26    /* deprecated */
+#define TAG_LCDAUTOADJMINAC		MOD_DISPLAY | 27    /* deprecated */
+#define TAG_LCDAUTOADJMAXAC		MOD_DISPLAY | 28    /* deprecated */
 #define TAG_KBDAUTOADJUST		MOD_DISPLAY | 13
 #define TAG_KBDBRIGHTNESS		MOD_DISPLAY | 21
 #define TAG_KBDONBRIGHTNESS		MOD_DISPLAY | 29
-//#define TAG_KBDBRIGHTNESSMAX	MOD_DISPLAY | 22    /* deprecated */
 #define TAG_KBDFADINGSPEED		MOD_DISPLAY | 23
-#define TAG_KBDTHRESHOLD		MOD_DISPLAY | 14
+#define TAG_KBDTHRESHOLD		MOD_DISPLAY | 14    /* deprecated */
 #define TAG_BRIGHTNESSOP		MOD_DISPLAY | FLG_PRIVATE | 6
 #define TAG_FRAMEBUFFERDEVICE	MOD_DISPLAY | FLG_STRING | 8
 #define TAG_BLANKFRAMEBUFFER	MOD_DISPLAY | 9
@@ -112,23 +108,30 @@
 #define TAG_CRTMIRROR			MOD_DISPLAY | 30
 #define TAG_CRTMIRRORKEY		MOD_DISPLAY | 31
 #define TAG_CRTMIRRORMOD		MOD_DISPLAY | 32
+#define TAG_BACKLIGHTDRIVER		MOD_DISPLAY | FLG_STRING | 33
+#define TAG_LCDAUTOADJBAT		MOD_DISPLAY | 34
+#define TAG_LCDAUTOADJAC		MOD_DISPLAY | 35
+#define TAG_KBDAUTOADJBAT		MOD_DISPLAY | 36
+#define TAG_KBDAUTOADJAC		MOD_DISPLAY | 37
 
 #define MOD_MIXER				5 << 16
-#define TAG_MIXERDEVICE			MOD_MIXER | FLG_STRING | 1
+#define TAG_MIXER				MOD_MIXER | FLG_STRING | 1
+#define TAG_MIXERDEVICE			TAG_MIXER                   /* deprecated */
 #define TAG_VOLUMEUPKEY			MOD_MIXER | 2
 #define TAG_VOLUMEUPMOD			MOD_MIXER | 3
 #define TAG_VOLUMEDOWNKEY		MOD_MIXER | 4
 #define TAG_VOLUMEDOWNMOD		MOD_MIXER | 5
 #define TAG_MUTEKEY				MOD_MIXER | 6
 #define TAG_MUTEMOD				MOD_MIXER | 7
-#define TAG_MIXERINITDELAY		MOD_MIXER | 8
+#define TAG_MIXERINITDELAY		MOD_MIXER | 8               /* deprecated */
 #define TAG_VOLUME				MOD_MIXER | 9
 #define TAG_MUTE				MOD_MIXER | 10
 #define TAG_MIXERCHANNELS		MOD_MIXER | FLG_STRING | 11
-#define TAG_SOUNDSYSTEM			MOD_MIXER | 12
-#define TAG_SOUNDSYSTEMCFG		MOD_MIXER | 13
-#define TAG_MIXERCARD			MOD_MIXER | FLG_STRING | 14
-#define TAG_MIXERELEMENTS		MOD_MIXER | FLG_STRING | 15
+#define TAG_SOUNDSYSTEM			MOD_MIXER | 12              /* deprecated */
+#define TAG_SOUNDSYSTEMCFG		MOD_MIXER | 13              /* deprecated */
+#define TAG_MIXERCARD			MOD_MIXER | FLG_STRING | 14 /* deprecated */
+#define TAG_MIXERELEMENTS		MOD_MIXER | FLG_STRING | 15 /* deprecated */
+#define TAG_MIXERDRIVER			MOD_MIXER | FLG_STRING | 16
 
 #define MOD_CDROM				6 << 16
 #define TAG_CDROMDEVICE			MOD_CDROM | FLG_STRING | 1
