@@ -88,16 +88,18 @@ static struct tagtable tagtab[] = {
   { "TAG_LCDBRIGHTNESS",     TAG_LCDBRIGHTNESS},
   { "TAG_LCDFADINGSPEED",    TAG_LCDFADINGSPEED},
   { "TAG_LCDAUTOADJUST",     TAG_LCDAUTOADJUST },
-  { "TAG_LCDTHRESHOLD",      TAG_LCDTHRESHOLD },
-  { "TAG_LCDAUTOADJMINBAT",  TAG_LCDAUTOADJMINBAT },
-  { "TAG_LCDAUTOADJMAXBAT",  TAG_LCDAUTOADJMAXBAT },
-  { "TAG_LCDAUTOADJMINAC",   TAG_LCDAUTOADJMINAC },
-  { "TAG_LCDAUTOADJMAXAC",   TAG_LCDAUTOADJMAXAC },
+#ifdef DEBUG
+  { "TAG_LCDAUTOADJAC",      TAG_LCDAUTOADJAC },
+  { "TAG_LCDAUTOADJBAT",     TAG_LCDAUTOADJBAT },
+#endif
   { "TAG_KBDBRIGHTNESS",     TAG_KBDBRIGHTNESS},
   { "TAG_KBDONBRIGHTNESS",   TAG_KBDONBRIGHTNESS},
   { "TAG_KBDFADINGSPEED",    TAG_KBDFADINGSPEED},
-  { "TAG_KBDTHRESHOLD",      TAG_KBDTHRESHOLD },
   { "TAG_KBDAUTOADJUST",     TAG_KBDAUTOADJUST },
+#ifdef DEBUG
+  { "TAG_KBDAUTOADJAC",      TAG_KBDAUTOADJAC },
+  { "TAG_KBDAUTOADJBAT",     TAG_KBDAUTOADJBAT },
+#endif
   { "TAG_BRIGHTNESSOP",      TAG_BRIGHTNESSOP},
   { "TAG_FRAMEBUFFERDEVICE", TAG_FRAMEBUFFERDEVICE},
   { "TAG_BLANKFRAMEBUFFER",  TAG_BLANKFRAMEBUFFER},
@@ -166,7 +168,7 @@ static struct tagtable tagtab[] = {
   { N_("Permission denied"),          E_PERM},
   { N_("Private Tag"),                E_PRIVATE},
   { N_("File doesn't exist"),         E_NOEXIST},
-  { N_("File not a charakter device"),E_NOCHAR},
+  { N_("File not a character device"),E_NOCHAR},
   { N_("File not a block device"),    E_NOBLK},
   { N_("File not a file"),            E_NOFILE},
   { N_("Buffer overflow"),            E_BUFOVL},
@@ -177,7 +179,7 @@ static struct tagtable tagtab[] = {
   { N_("Help or version info"),         E_INFO},
   { N_("Server not found"),               E_NOSERVER},
   { N_("Registration failed"),              E_REGISTER},
-  { N_("Unsecure script owner"),             E_USER},
+  { N_("Insecure script owner"),             E_USER},
   { N_("Script must be write-only by owner"), E_RIGHTS},
   { N_("read-only value"),			E_NOWRITE},
   { N_("write-only value"),	  		E_NOREAD},
