@@ -23,6 +23,7 @@
 #define CMD_FLAGS		1
 #define CMD_QUERY		2
 #define CMD_CONFIG		3
+#define CMD_LISTCLIENTS	4
 
 struct tagtable {
 	char *tagname;
@@ -33,10 +34,12 @@ struct tagtable {
 
 /* private prototypes */
 void print_usage (char *prgname);
+void print_cmds ();
 void print_tags ();
 int evaluate_args(int argc, char *argv[], struct tagitem *taglist);
 int read_message (char *buffer, int bufferlen);
 long identifytag (char *name);
+long identifycmd (char *name);
 char *identifytagname (long tag, char *defname);
 char *identifyerror (long error, char *deferror);
 
