@@ -57,7 +57,7 @@ typedef struct inputsource {
 /* public prototypes */
 int inputmanager_init ();
 void cbFreeInputSource (gpointer data, gpointer user_data);
-int inputmanager_exit ();
+void inputmanager_exit ();
 int track_modifier(unsigned short code, unsigned int value);
 int get_modifier();
 
@@ -65,13 +65,13 @@ gint cbEventDevices (gconstpointer a, gconstpointer b);
 void scanEventDevices ();
 void destroyInputSource (gpointer data);
 gboolean handleInputSource (GIOChannel *io, GIOCondition condition, gpointer data);
-InputSource *addInputSource (int fd, void *handler, gpointer user_data, gboolean close_on_exit);
-int register_function (int queueid, void *func);
+InputSource *addInputSource (int fd, int (*handler)(), gpointer user_data, gboolean close_on_exit);
+int register_function (int queueid, void (*func)());
 long process_queue_single (int queueid, long tag, long data);
 int process_queue (int queueid, struct tagitem *taglist);
 void ipc_handler ();
-gboolean event_handler (int fd, gpointer user_data);
-gboolean mice_handler (int fd, gpointer user_data);
+gboolean input_event_handler (int fd, gpointer user_data);
+/*gboolean mice_handler (int fd, gpointer user_data);*/
 gboolean cb_timer100 (gpointer data);
 gboolean cb_timer1000 (gpointer data);
 
