@@ -1,15 +1,18 @@
-/* --------------------------------------------------------------------------
- * module_cdrom.c
- * code for module cdrom
+/**
+ * @brief Module to eject the CDROM on users request
  *
- * Copyright 2002 Matthias Grimm
+ * This module contains the code to read a key from the keyboard and
+ * eject the CDROM on trigger. This module is most usefull on some
+ * Apple PowerBooks that have no eject key on the drive.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version
- * 2 of the License, or (at your option) any later version.
- * -------------------------------------------------------------------------*/
-
+ * version 2 as published by the Free Software Foundation
+ * (http://www.gnu.org/licenses/gpl.html)
+ *
+ * @file    src/module_cdrom.c
+ * @author  Matthias Grimm <matthias.grimm@users.sourceforge.net>
+ */
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
@@ -28,11 +31,14 @@
 #include "pbbinput.h"
 #include "gettext_macros.h"
 #include "input_manager.h"
-#include "config_manager.h"
 #include "module_cdrom.h"
 #include "support.h"
 #include "debug.h"
 
+#include "class_config.h"
+
+#define SECTION "MODULE CDROM"
+
 struct moddata_cdrom {
 	char *cdromdev;		/* pathname of the cdrom device */
 	unsigned short keyejectcd;
@@ -46,63 +52,34 @@ int
 cdrom_init ()
 {
 	struct moddata_cdrom *base = &modbase_cdrom;
-	static char devbuffer_cdrom[STDBUFFERLEN];
-	int sid;
-
-	sprintf(devbuffer_cdrom, DEFAULT_CDROM);
-
-	base->cdromdev		= devbuffer_cdrom;
-	base->keyejectcd	= KEY_EJECTCD;
-	base->modejectcd	= MOD_NONE;
-	base->keydelaytime	= 0;
-
-	if ((sid = registerCfgSection ("MODULE CDROM")) == 0) {
-		print_msg (PBB_ERR, _("Can't register configuration section %s, out of memory."), "MODULE CDROM");
-		return E_NOMEM;
-	} else {
-		registerCfgOptionString (sid, "dev_CDROM", TAG_CDROMDEVICE, 0,
-				NULL);
-		registerCfgOptionKey (sid, "EjectCDKey", TAG_EJECTCDKEY, TAG_EJECTCDMOD, 0,
-				NULL);
-		registerCfgOptionInt (sid, "EjectCDKeyDelay", TAG_EJECTCDKEYDELAY, 0,
-				NULL);
-	}
-
-	register_function (QUERYQUEUE, cdrom_query);
-	register_function (CONFIGQUEUE, cdrom_configure);
-	return 0;
-}
-
-int
-cdrom_open (struct tagitem *taglist)
-{
-	struct moddata_cdrom *base = &modbase_cdrom;
-	char *devname;
 	int rc;
 
-	devname = (char *) tagfind (taglist, TAG_CDROMDEVICE, (long) DEFAULT_CDROM);
-	if ((rc = copy_path (devname, base->cdromdev, TYPE_BLKDEV, CPFLG_MAYBEMISSED))) {
+	/* get module configuration */
+	base->cdromdev     = config_get_string  (SECTION, "Device",          DEFAULT_CDROM);
+	base->keyejectcd   = config_get_keycode (SECTION, "EjectCDKey",      KEY_EJECTCD);
+	base->modejectcd   = config_get_modmask (SECTION, "EjectCDKey",      MOD_NONE);
+	base->keydelaytime = config_get_int     (SECTION, "EjectCDKeyDelay", 0);
+
+	if ((rc = check_path (base->cdromdev, TYPE_BLKDEV, CPFLG_MAYBEMISSED))) {
 		print_msg (PBB_ERR, _("Please check your CDROM settings in configuration file.\n"));
 		return rc;
 	}
-		
+
 #if defined(DEBUG) && CDROMPERF
 	cdrom_testperformance ();
 #endif	
+	register_function (QUERYQUEUE, cdrom_query);
+	register_function (CONFIGQUEUE, cdrom_configure);
 	register_function (KBDQUEUE, cdrom_keyboard);
 	return 0;
 }
 
-int
-cdrom_close ()
-{
-	return 0;
-}
-
-int
+void
 cdrom_exit ()
 {
-	return 0;
+	struct moddata_cdrom *base = &modbase_cdrom;
+
+	g_free (base->cdromdev);
 }
 
 void
@@ -156,22 +133,34 @@ cdrom_handle_tags (int cfgure, struct tagitem *taglist)
 			break;
 		case TAG_CDROMDEVICE:
 			if (cfgure) {
-				if ((err = copy_path ((char *) taglist->data, base->cdromdev, TYPE_BLKDEV, CPFLG_MAYBEMISSED)))
+				if ((err = check_path ((char *) taglist->data, TYPE_BLKDEV, CPFLG_MAYBEMISSED)))
 					tagerror (taglist, err);
+				else {
+					replace_string (&base->cdromdev, (char *) taglist->data);
+					config_set_string (SECTION, "Device", base->cdromdev);
+				}
 			} else
 				taglist->data = (long) base->cdromdev;
 			break;
 		case TAG_EJECTCDKEY:
-			if (cfgure)	base->keyejectcd = taglist->data;
-			else		taglist->data = base->keyejectcd;
+			if (cfgure)	{
+				base->keyejectcd = taglist->data;
+				config_set_keymod (SECTION, "EjectCDKey", taglist->data, base->modejectcd);
+			} else
+				taglist->data = base->keyejectcd;
 			break;
 		case TAG_EJECTCDMOD:
-			if (cfgure)	base->modejectcd = taglist->data;
-			else		taglist->data = base->modejectcd;
+			if (cfgure) {
+				base->modejectcd = taglist->data;
+				config_set_keymod (SECTION, "EjectCDKey", base->keyejectcd, taglist->data);
+			} else
+				taglist->data = base->modejectcd;
 			break;
 		case TAG_EJECTCDKEYDELAY:
-			if (cfgure)	base->keydelaytime = taglist->data;
-			else		taglist->data = base->keydelaytime;
+			if (cfgure)	{
+				base->keydelaytime = taglist->data;
+				config_set_int (SECTION, "EjectCDKeyDelay", taglist->data);
+			} else		taglist->data = base->keydelaytime;
 			break;
 		}
 		taglist++;
@@ -313,8 +302,11 @@ cdrom_eject()
 	char *mp;
 
 	singletag_to_clients(CHANGEVALUE, TAG_EJECTCD, 0); /* busy */
-	if ((mp = cdrom_getmountpoint(base->cdromdev)))
-		err = call_script ("/bin/umount %.30s", mp);
+	if ((mp = cdrom_getmountpoint(base->cdromdev))) {
+		err = call_script ("/usr/bin/eject %.30s", mp);
+		if (err)
+		    err = call_script ("/bin/umount %.30s", mp);
+	}
 
 #if defined(DEBUG) && CDROM
 	printf ("mp=%s, err=%d, %s (%d)\n", mp, err, strerror(errno), errno);
