@@ -18,9 +18,7 @@
 
 /* public prototypes */
 int cdrom_init ();
-int cdrom_open (struct tagitem *taglist);
-int cdrom_close ();
-int cdrom_exit ();
+void cdrom_exit ();
 
 /* private prototypes */
 void cdrom_keyboard (struct tagitem *taglist);
