@@ -1,16 +1,16 @@
 #ifndef INCLUDE_MODULE_DISPLAY_H
 #define INCLUDE_MODULE_DISPLAY_H
-/* ----------------------------------------------------------------------------
- * module_display.h
- * prototypes and definitions for the module display.
- *
- * Copyright 2002-2005 Matthias Grimm
- *
+/**
+ * @brief Definitions and prototypes for module_display.c
+ * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version
- * 2 of the License, or (at your option) any later version.
- * ----------------------------------------------------------------------------*/
+ * version 2 as published by the Free Software Foundation
+ * (http://www.gnu.org/licenses/gpl.html)
+ *
+ * @file    src/module_display.h
+ * @author  Matthias Grimm <matthias.grimm@users.sourceforge.net>
+ */
 
 #include <pbbtaglist.h>
 #include "debug.h"
@@ -45,6 +45,8 @@
 #define FBIO_ATY128_GET_MIRROR	_IOR('@', 1, __u32)
 #define FBIO_ATY128_SET_MIRROR	_IOW('@', 2, __u32)
 
+enum {BATTERY, AC};
+enum {AUTOADJ_OFF, AUTOADJ_LIN, AUTOADJ_HYS, AUTOADJ_LAST=AUTOADJ_HYS};
 
 /* operation codes (OP) for display_setlevel() */
 #define OP_DIM_LIGHT	1
@@ -63,8 +65,6 @@
 
 struct display_flags {
 	unsigned int lmu_enable:1;
-	unsigned int lcd_autoadjust:1;
-	unsigned int kbd_autoadjust:1;
 	unsigned int kbd_switchedoff:1;
 	unsigned int ctrl_fb:1;	   /* 1, if framebuffer should be blanked too */
 	unsigned int dimfullydark:1;
@@ -74,6 +74,13 @@ struct display_flags {
 	unsigned int :0;
 };
 
+struct autoadj_param {
+	unsigned int ambient1;  /* x1 */
+	unsigned int level1;    /* y1 */
+	unsigned int ambient2;  /* x2 */
+	unsigned int level2;    /* y2 */
+};
+
 struct display_light {
 	int current;     /* current brightness value */
 	int target;      /* brightness to set to */
@@ -82,7 +89,10 @@ struct display_light {
 	int offset;      /* users influence on automatic adjustment */
 	int fading;      /* timer interval or 0 for no fading */
 	int fadingstep; 
-	int threshold;   /* ambient threshold for automatic adjustment */
+	int autoadj_mode; /* operating mode of the ambient controller: off, linear or hysteresis */
+	unsigned int autoadj_hyst_out; /* the hysteresis function needs to know its output status */
+	struct autoadj_param ac_params;
+	struct autoadj_param battery_params;
 	int isrunning;   /* display timer currently running */
 #if defined(DEBUG) && DISPLAYTIMER
 	struct timeval start;  /* for profiling fading routines */
@@ -92,9 +102,7 @@ struct display_light {
 
 /* public prototypes */
 int display_init ();
-int display_open (struct tagitem *taglist);
-int display_close ();
-int display_exit ();
+void display_exit ();
 
 /* private prototypes */
 void display_keyboard (struct tagitem *taglist);
@@ -104,11 +112,15 @@ void display_configure (struct tagitem *taglist);
 gboolean display_timer (gpointer data);
 void display_ambienttimer (struct tagitem *taglist);
 void display_handle_tags (int mode, struct tagitem *taglist);
+int display_calc_brightness (struct display_light *illu, int pwrsource, int ambient, int ambientmax);
 int display_set_lcdbrightness(int level);
 int display_set_kbdbrightness(int level);
 int display_clip_brightness(struct display_light *illu, int level);
 void display_set_fading (struct display_light *illu, int fading);
 int display_get_fading (struct display_light *illu);
+void display_set_autoadj_from_tag (struct display_light *illu, int pwrsource, tag_t data);
+tag_t display_get_autoadj_for_tag (struct display_light *illu, int pwrsource);
+void display_set_autoadj_parameter (struct display_light *illu, int pwrsource, int *params);
 void display_change_brightness(int op);
 void display_framebuffer (int on);
 void display_sync_lcdbrightness (void);
