@@ -36,7 +36,6 @@ static struct tagtable tagtab[] = {
   { "TAG_USERALLOWED",       TAG_USERALLOWED},
   { "TAG_AUTORESCAN",        TAG_AUTORESCAN},
   { "TAG_TIMEFORCMD",        TAG_TIMEFORCMD},
-  { "TAG_CFGNAME",           TAG_CFGNAME},
   { "TAG_VERSION",           TAG_VERSION},
   { "TAG_REINIT",            TAG_REINIT}, 
   { "TAG_KEYCODE",           TAG_KEYCODE},
@@ -48,7 +47,6 @@ static struct tagtable tagtab[] = {
   { "TAG_SYSINFOSLAVE",      TAG_SYSINFOSLAVE},
   { "TAG_BACKLIGHTMAX",      TAG_BACKLIGHTMAX},
   { "TAG_BACKLIGHTLEVEL",    TAG_BACKLIGHTLEVEL},
-  { "TAG_HW_BACKLIGHT",      TAG_HW_BACKLIGHT},
   { "TAG_COVERSTATUS",       TAG_COVERSTATUS},
   { "TAG_BATLOG",            TAG_BATLOG},
   { "TAG_BATCYCLE",          TAG_BATCYCLE},
@@ -65,6 +63,7 @@ static struct tagtable tagtab[] = {
   { "TAG_AMBIENTLIGHT",      TAG_AMBIENTLIGHT},
   { "TAG_BATTERYPRESENT",    TAG_BATTERYPRESENT},
 #endif
+  { "TAG_AMBIENTLIGHTMAX",   TAG_AMBIENTLIGHTMAX},
   { "TAG_POWERCHANGED",      TAG_POWERCHANGED},
   { "TAG_REQUESTSLEEP",      TAG_REQUESTSLEEP},
   { "TAG_SLEEPSUPPORTED",    TAG_SLEEPSUPPORTED},
@@ -92,18 +91,16 @@ static struct tagtable tagtab[] = {
   { "TAG_KBDILLUMONKEY",     TAG_KBDILLUMONKEY},
   { "TAG_KBDILLUMONMOD",     TAG_KBDILLUMONMOD},
   { "TAG_LCDAUTOADJUST",     TAG_LCDAUTOADJUST },
+  { "TAG_LCDAUTOADJAC",      TAG_LCDAUTOADJAC },
+  { "TAG_LCDAUTOADJBAT",     TAG_LCDAUTOADJBAT },
   { "TAG_LCDBRIGHTNESS",     TAG_LCDBRIGHTNESS},
   { "TAG_LCDFADINGSPEED",    TAG_LCDFADINGSPEED},
-  { "TAG_LCDTHRESHOLD",      TAG_LCDTHRESHOLD },
-  { "TAG_LCDAUTOADJMINBAT",  TAG_LCDAUTOADJMINBAT },
-  { "TAG_LCDAUTOADJMAXBAT",  TAG_LCDAUTOADJMAXBAT },
-  { "TAG_LCDAUTOADJMINAC",   TAG_LCDAUTOADJMINAC },
-  { "TAG_LCDAUTOADJMAXAC",   TAG_LCDAUTOADJMAXAC },
   { "TAG_KBDAUTOADJUST",     TAG_KBDAUTOADJUST },
+  { "TAG_KBDAUTOADJAC",      TAG_KBDAUTOADJAC },
+  { "TAG_KBDAUTOADJBAT",     TAG_KBDAUTOADJBAT },
   { "TAG_KBDBRIGHTNESS",     TAG_KBDBRIGHTNESS},
   { "TAG_KBDONBRIGHTNESS",   TAG_KBDONBRIGHTNESS},
   { "TAG_KBDFADINGSPEED",    TAG_KBDFADINGSPEED},
-  { "TAG_KBDTHRESHOLD",      TAG_KBDTHRESHOLD },
   { "TAG_BRIGHTNESSOP",      TAG_BRIGHTNESSOP},
   { "TAG_FRAMEBUFFERDEVICE", TAG_FRAMEBUFFERDEVICE},
   { "TAG_BLANKFRAMEBUFFER",  TAG_BLANKFRAMEBUFFER},
@@ -173,7 +170,7 @@ static struct tagtable tagtab[] = {
   { N_("Permission denied"),           E_PERM},
   { N_("Private Tag"),                 E_PRIVATE},
   { N_("File doesn't exist"),          E_NOEXIST},
-  { N_("File not a charakter device"), E_NOCHAR},
+  { N_("File not a character device"), E_NOCHAR},
   { N_("File not a block device"),     E_NOBLK},
   { N_("File not a file"),             E_NOFILE},
   { N_("Buffer overflow"),             E_BUFOVL},
@@ -182,7 +179,7 @@ static struct tagtable tagtab[] = {
   { N_("Messageport not available"),    E_MSGPORT},
   { N_("Server already running"),         E_TWICE},
   { N_("Help or version info"),             E_INFO},
-  { N_("Unsecure script owner"),             E_USER},
+  { N_("Insecure script owner"),             E_USER},
   { N_("Script must be write-only by owner"), E_RIGHTS},
   { N_("read-only value"),			E_NOWRITE},
   { N_("write-only value"), 			E_NOREAD},
@@ -199,12 +196,6 @@ struct moddata_peep {
 
 int
 peep_init ()
-{
-	return peep_open (NULL);
-}
-
-int
-peep_open (struct tagitem *taglist)
 {
 	struct moddata_peep *base = &modbase_peep;
 
@@ -230,10 +221,9 @@ peep_open (struct tagitem *taglist)
 	return 0;
 }
 
-int
+void
 peep_exit ()
 {
-  return 0;
 }
 
 #if PEEP_KEYBOARD
@@ -338,7 +328,7 @@ peep_taglist (char *source, struct tagitem *taglist)
 					mark = ' ';
 
 				printf ("%-10s: %c %-25s: ", source, mark, tt->tagname);
-				if (taglist->data == 0xffffffff) {
+				if (taglist->data == -1) {
 					printf ("tag not filled");	
 				} else if (tt->tag == TAG_SKIP) {
 					tt2 = findttentry (tagtab,  taglist->data);
