@@ -21,8 +21,7 @@ struct tagtable {
 
 /* public prototypes */
 int peep_init ();
-int peep_open (struct tagitem *taglist);
-int peep_exit ();
+void peep_exit ();
 
 /* private prototypes */
 void peep_keyboard (struct tagitem *taglist);
