@@ -1,21 +1,25 @@
-/* --------------------------------------------------------------------------
- * module_pmac.c
- * code for test module
+/**
+ * @brief Hardware abstraction layer for PMU based machines
  *
- * Copyright 2002 Matthias Grimm
+ * This module contains all hardware related functions for PMU based
+ * machines that are used by the high level logic modules to do their
+ * job.
+ *
+ * This module is needed for all powerpc based Apple PowerBooks and
+ * iBooks.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version
- * 2 of the License, or (at your option) any later version.
- * -------------------------------------------------------------------------*/
+ * version 2 as published by the Free Software Foundation
+ * (http://www.gnu.org/licenses/gpl.html)
+ *
+ * @file    src/module_pmac.c
+ * @author  Matthias Grimm <matthias.grimm@users.sourceforge.net>
+ */
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
-#include "systems.h"
-
-#ifdef WITH_MODULE_PMAC
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -35,7 +39,7 @@
 
 #include "gettext_macros.h"
 #include "input_manager.h"
-#include "config_manager.h"
+#include "class_config.h"
 #include "module_pmac.h"
 #include "support.h"
 #ifdef WITH_PMUD
@@ -44,18 +48,18 @@
 #include "debug.h"
 
 #define TRACKPAD_PAUSETIME	6	/* time in 1/100 of a second */
+#define SECTION "MODULE PMAC"
 
 struct moddata_pmac {
 	char *pmudev;    /* pathname of the PMU device */
 	int fd_pmu;      /* filehandle of the PMU device */
 	char *adbdev;    /* pathname of the ADB device */
 	int fd_adb;      /* filehandle of ADB device */
-	char *i2cdev;    /* pathname of the I2C device with ambient sensor and KBD illu */
-
-	int version;     /* PMU version */
-	char *identity;  /* Identity string of this laptop */
-	int machine;     /* OpenFirmware machine code */
-	int lmuaddr;	 /* i2c address of LMU Controller */
+	GString *i2cdev;   /* pathname of the I2C device with ambient sensor and KBD illu */
+	GString *identity; /* Identity string of this laptop */
+	int version;       /* PMU version */
+	int machine;       /* OpenFirmware machine code */
+	int lmuaddr;	   /* i2c address of LMU Controller */
 	struct modflags_pmac flags;
 	int flagschanged;        /* PMU flags that have changed recently */
 	int oharevolbutton;      /* level of volume button on OHARE PBs scaled to 0..100 */
@@ -78,139 +82,87 @@ struct moddata_pmac {
 	int keyboard_mode;             /* keyboard mode for recent keyboards (0xC[0..4]) */
 	int batlog_mode;
 	int batlog_cycle;
-	int lcdfeedback;               /* correction factor for ambient light sensor */
-#if defined(DEBUG) && SIMUAMBIENT
+#if defined(DEBUG) && SIMU_AMBIENT
 	int ambient;
 #endif
 } modbase_pmac;
 
+static const char *TPModeList[] = { "notap", "tap", "drag", "lock", NULL };
+static const char *KBDModeList[] = { "disabled", "fkeyslast", "fkeysfirst", NULL };
+static const char *BatlogModeList[] = { "none", "cycle", "log", NULL };
+
+/**
+ * @brief Initalizes the module pmac
+ *
+ * This function initializes the module pmac. The data structure will
+ * be initialized and the hardware status is read.
+ *
+ * Immideately after this initialisation the module is able to and 
+ * will answer upon requests.
+ *
+ * @return  error code, zero if the module is ready to operate
+ */
 int
 pmac_init ()
 {
 	struct moddata_pmac *base = &modbase_pmac;
-	static char devbuffer_pmu[STDBUFFERLEN];
-	static char devbuffer_adb[STDBUFFERLEN];
-	static char devbuffer_i2c[STDBUFFERLEN];
-	static char identitystring[STDBUFFERLEN];
-	int sid;
-	
-	bzero (base, sizeof (struct moddata_pmac));
-	bzero (devbuffer_pmu, STDBUFFERLEN);
-	bzero (devbuffer_adb, STDBUFFERLEN);
-	bzero (devbuffer_i2c, STDBUFFERLEN);
-	bzero (identitystring, STDBUFFERLEN);
-	base->pmudev   = devbuffer_pmu;
-	base->adbdev   = devbuffer_adb;
-	base->i2cdev   = devbuffer_i2c;
-	base->identity = identitystring;
-
-	if ((sid = registerCfgSection ("MODULE PMAC")) == 0) {
-		print_msg (PBB_ERR, _("Can't register configuration section %s, out of memory."), "MODULE PMAC");
-		return E_NOMEM;
-	} else {
-		registerCfgOptionString (sid, "dev_PMU", TAG_PMUDEVICE, 0,
-				NULL);
-		registerCfgOptionString (sid, "dev_ADB", TAG_ADBDEVICE, 0,
-				NULL);
-		registerCfgOptionKey (sid, "TPModeUpKey", TAG_TPMODEUPKEY, TAG_TPMODEUPMOD, 0,
-				NULL);
-		registerCfgOptionKey (sid, "TPModeDownKey", TAG_TPMODEDOWNKEY, TAG_TPMODEDOWNMOD, 0,
-				NULL);
-		registerCfgOptionList (sid, "TPMode", TAG_TPMODE, 0,
-				N_("'"TRACKPAD_NOTAP_NAME"', '"TRACKPAD_TAP_NAME"', '"TRACKPAD_DRAG_NAME"' or '"TRACKPAD_LOCK_NAME"'"),
-				TRACKPAD_NOTAP_NAME, TRACKPAD_TAP_NAME, TRACKPAD_DRAG_NAME, TRACKPAD_LOCK_NAME, NULL);
-		registerCfgOptionList (sid, "KBDMode", TAG_KBDMODE, 0,
-				N_("'"KEYBOARD_FNDISABLED_NAME"', '"KEYBOARD_FNBACK_NAME"' or '"KEYBOARD_FNTOP_NAME"'"),
-				KEYBOARD_FNDISABLED_NAME, KEYBOARD_FNBACK_NAME, KEYBOARD_FNTOP_NAME, NULL);
-		registerCfgOptionList (sid, "Batlog", TAG_BATLOG, 0,
-				N_("'"BATLOG_NONE_NAME"', '"BATLOG_CYCLE_NAME"' or '"BATLOG_LOG_NAME"'"),
-				BATLOG_NONE_NAME, BATLOG_CYCLE_NAME, BATLOG_LOG_NAME, NULL);
-		registerCfgOptionBool (sid, "NoTapTyping", TAG_NOTAPTYPING, 0,
-				N_("switch trackpad to 'notap-mode' while typing"));
-	}
-
-	register_function (QUERYQUEUE, pmac_query);
-	register_function (CONFIGQUEUE, pmac_configure);
-	return 0;
-}
-
-
-int
-pmac_open (struct tagitem *taglist)
-{
-	struct moddata_pmac *base = &modbase_pmac;
-	char *devname;
 	long DUMMY = 0;
 	int val, rc;
-
-	base->fd_pmu		    = -1;
-	base->fd_adb		     = -1;
-	base->flags.coveropen     = 1;         /* lid is open */
-	base->flags.sleepsupported = 0;        /* sleep not supported by default */
-	base->flags.set_kbdmode  = 0;          /*    request flags for delayed actions */
-	base->flags.goto_sleep   = 0;          /*  /  */
-	base->flags.notap_typing = 0;
-
-	base->keytpmodeup	= KEY_BRIGHTNESSUP;
-	base->modtpmodeup	= MOD_ALT;
-	base->keytpmodedn	= KEY_BRIGHTNESSDOWN;
-	base->modtpmodedn	= MOD_ALT;
-	base->trackpad_mode	= -1;
-	base->trackpad_pause = 0;
-	base->keyboard_mode	= -1;
-	base->batlog_mode	= BATLOG_NONE;  /* no batlog */
-	base->batlog_cycle	= -1;         /* cycle invalid */
-	base->timeleft      = -1;
-	base->machine       =  0;
-#if defined(DEBUG) && SIMUAMBIENT
-	base->ambient       = 0;
-#endif
-#ifdef WITH_PMUD
-	print_msg (PBB_INFO, _("pmud support compiled in and active.\n"));
+	
+	base->i2cdev         = g_string_new(NULL);
+	base->identity       = g_string_new(NULL);
+	base->fd_pmu         = -1;
+	base->fd_adb         = -1;
+	base->trackpad_pause =  0;
+	base->batlog_cycle	 = -1;  /* cycle invalid */
+	base->timeleft       = -1;
+	base->machine        =  0;
+#if defined(DEBUG) && SIMU_AMBIENT
+	base->ambient        =  0;
 #endif
+	base->flags.coveropen      = 1;   /* lid is open */
+	base->flags.sleepsupported = 0;   /* sleep not supported by default */
+	base->flags.set_kbdmode    = 0;   /*   request flags for delayed actions */
+	base->flags.goto_sleep     = 0;   /*  /  */
 
-	devname = (char *) tagfind (taglist, TAG_PMUDEVICE, (long) DEFAULT_PMU);
-	if ((rc = copy_path (devname, base->pmudev, TYPE_CHARDEV, CPFLG_NONE)) == 0) {
+	base->pmudev       = config_get_string  (SECTION, "Device_PMU",    DEFAULT_PMU);
+	base->adbdev       = config_get_string  (SECTION, "Device_ADB",    DEFAULT_ADB);
+	base->keytpmodeup  = config_get_keycode (SECTION, "TPModeUpKey",   KEY_BRIGHTNESSUP);
+	base->modtpmodeup  = config_get_modmask (SECTION, "TPModeUpKey",   MOD_ALT);
+	base->keytpmodedn  = config_get_keycode (SECTION, "TPModeDownKey", KEY_BRIGHTNESSDOWN);
+	base->modtpmodedn  = config_get_modmask (SECTION, "TPModeDownKey", MOD_ALT);
+	base->batlog_mode	= config_get_option (SECTION, "Batlog", BatlogModeList, BATLOG_NONE);
+	base->flags.notap_typing = config_get_boolean (SECTION, "NoTapTyping", FALSE);
+
+	if ((rc = check_path (base->pmudev, TYPE_CHARDEV, CPFLG_NONE)) == 0) {
 		if ((base->fd_pmu = open(base->pmudev, O_RDWR)) < 0) {
-			print_msg (PBB_ERR, _("Can't open PMU device %s: %s\n"), devname, strerror(errno));
+			print_msg (PBB_ERR, _("Can't open PMU device %s: %s\n"), base->pmudev, strerror(errno));
 			return E_OPEN;
 		}
 	} else return rc;
 
-	devname = (char *) tagfind (taglist, TAG_ADBDEVICE, (long) DEFAULT_ADB);
-	if ((rc = copy_path (devname, base->adbdev, TYPE_CHARDEV, CPFLG_NONE)) == 0) {
+	if ((rc = check_path (base->adbdev, TYPE_CHARDEV, CPFLG_NONE)) == 0) {
 		if ((base->fd_adb = open(base->adbdev, O_RDWR)) < 0) {
-			print_msg (PBB_ERR, _("Can't open ADB device %s: %s\n"), devname, strerror(errno));
+			print_msg (PBB_ERR, _("Can't open ADB device %s: %s\n"), base->adbdev, strerror(errno));
 			return E_OPEN;
 		}
 	} else return rc;
 
 	base->version = get_pmu_version(base->fd_adb);
 	base->machine = getMachineID (base->version);
-	snprintf(base->identity, STDBUFFERLEN, "%s - PMU version: %d",
+	g_string_printf (base->identity, "%s - PMU version: %d",
 			getMachineName(base->machine), base->version);
 	
+	/* check if sleep is supported on this system */
 	ioctl(base->fd_pmu, PMU_IOC_CAN_SLEEP, &val);
-	base->flags.sleepsupported = val == 1 ? 1 : 0; /* check if sleep is supported on this system */
+	base->flags.sleepsupported = val == 1 ? 1 : 0;
 	
 	/* If this PowerBook has an ambient light sensor connected
 	 * to the I2C bus, search for the device and save  name and
 	 * bus address
 	 */
 	if ((base->lmuaddr = getLMUAddress()) != 0)
-		strncpy (base->i2cdev, findLMUDevice(base->lmuaddr), STDBUFFERLEN);
-
-	/* If we have an Ambient light sensor in this PowerBook, we
-	 * must measure the LCD feedback to this sensor in current
-	 * state. To do this we switch the backlight off for a moment
-	 * to measure the real ambient light level.
-	 */
-	if (haveI2CAmbient() || havePMUAmbient()) {  
-		print_msg (PBB_INFO, _("LMU found, ambient light sensor and keyboard illumination active.\n"));
-		val = process_queue_single (QUERYQUEUE, TAG_HW_BACKLIGHT, -1);
-		pmac_set_lcdbacklight (0);    /* measure LCD feedback */
-		pmac_set_lcdbacklight (val);  /* restore LCD backlight level */
-	}
+		g_string_assign (base->i2cdev, findLMUDevice(base->lmuaddr));
 
 	pmac_update_flags ();
 	pmac_update_batteryinfo ();
@@ -219,18 +171,22 @@ pmac_open (struct tagitem *taglist)
 #ifdef DEBUG
 	print_msg (PBB_INFO, "DBG: Machine:  %0x\n", base->machine);	
     print_msg (PBB_INFO, "DBG: Keyboard: %s\n",
-			haveADBKeyboard () ? "ADB" : "USB");
+			base->machine > 1 ? haveADBKeyboard () ? "ADB" : "USB" : "ADB (without config)");
     print_msg (PBB_INFO, "DBG: Trackpad: %s\n",
 			haveADBTrackpad () ? "ADB" : "USB");
     print_msg (PBB_INFO, "DBG: Ambient:  %s\n",
 			haveI2CAmbient () ? "I2C" : havePMUAmbient () ? "PMU" : "None");
 #endif
+#ifdef WITH_PMUD
+	print_msg (PBB_INFO, _("pmud support compiled in and active.\n"));
+#endif
 
-	base->trackpad_mode = trackpad_get_config(); /* get trackpad mode */
+	base->trackpad_mode	= config_get_option (SECTION, "TPMode", TPModeList, trackpad_get_config());
+	trackpad_set_config(base->trackpad_mode); /* set configured trackpad mode */
+	base->keyboard_mode	= config_get_option (SECTION, "KBDMode", KBDModeList, keyboard_get_config());
+	keyboard_set_config(base->keyboard_mode); /* set configured keyboard mode */
 
 	if (base->version != OHARE_PMU) {
-		base->keyboard_mode = keyboard_get_config();	/* get keyboard config*/
-
 		/* If the kernel has been compiled with CONFIG_PMAC_BACKLIGHT_LEGACY
 		 * the following ioctl is needed to disable the ancient backlight
 		 * control of the kernel. Otherwise the new sysfs interface to the
@@ -254,27 +210,36 @@ pmac_open (struct tagitem *taglist)
 	register_function (T100QUEUE, pmac_timer100);
 	register_function (T1000QUEUE, pmac_timer1000);
 	register_function (SECUREQUEUE, pmac_secure);
+	register_function (QUERYQUEUE, pmac_query);
+	register_function (CONFIGQUEUE, pmac_configure);
 	return 0;
 }
 
-int
-pmac_close ()
+/**
+ * @brief  Frees all ressources allocated from module pmac
+ *
+ * This function is usually called on program termination. It
+ * frees all allocated ressources and closes all open file
+ * handles.
+ *
+ * @return  always zero
+ */
+void
+pmac_exit ()
 {
 	struct moddata_pmac *base = &modbase_pmac;
 
 	batlog_save();            /* save batlog info to disc */
+
 	if (base->fd_pmu != -1)
 		close (base->fd_pmu); /* close PMU device */
 	if (base->fd_adb != -1)
 		close (base->fd_adb); /* close ADB device */
 
-	return 0;
-}
-
-int
-pmac_exit ()
-{
-	return 0;
+	g_free (base->pmudev);
+	g_free (base->adbdev);
+	g_string_free (base->identity, TRUE);
+	g_string_free (base->i2cdev, TRUE);
 }
 
 void
@@ -403,49 +368,69 @@ pmac_handle_tags (int cfgure, struct tagitem *taglist)
 			break;
 		case TAG_PMUDEVICE:
 			if (cfgure) {
-				if ((err = copy_path ((char *) taglist->data, base->pmudev, TYPE_CHARDEV, CPFLG_NONE)))
+				if ((err = check_path ((char *) taglist->data, TYPE_CHARDEV, CPFLG_NONE)))
 					tagerror (taglist, err);
+				else {
+					replace_string (&base->pmudev, (char *) taglist->data);
+					config_set_string (SECTION, "Device_PMU", base->pmudev);
+				}
 			} else
 				taglist->data = (long) base->pmudev;
 			break;
 		case TAG_ADBDEVICE:
 			if (cfgure) {
-				if ((err = copy_path ((char *) taglist->data, base->adbdev, TYPE_CHARDEV, CPFLG_NONE)))
+				if ((err = check_path ((char *) taglist->data, TYPE_CHARDEV, CPFLG_NONE)))
 					tagerror (taglist, err);
+				else {
+					replace_string (&base->adbdev, (char *) taglist->data);
+					config_set_string (SECTION, "Device_ADB", base->adbdev);
+				}
 			} else
 				taglist->data = (long) base->adbdev;
 			break;
-		case TAG_BACKLIGHTLEVEL:   /* private tag */
-			if (cfgure)	pmac_set_lcdbacklight (taglist->data);
-			else		taglist->data = process_queue_single (QUERYQUEUE, TAG_HW_BACKLIGHT, -1);
-			break;
 		case TAG_TPMODEUPKEY:
-			if (cfgure)	base->keytpmodeup = taglist->data;
-			else		taglist->data = base->keytpmodeup;
+			if (cfgure)	{
+				base->keytpmodeup = taglist->data;
+				config_set_keymod (SECTION, "TPModeUpKey", taglist->data, base->modtpmodeup);
+			} else
+				taglist->data = base->keytpmodeup;
 			break;
 		case TAG_TPMODEUPMOD:
-			if (cfgure)	base->modtpmodeup = taglist->data;
-			else		taglist->data = base->modtpmodeup;
+			if (cfgure)	{
+				base->modtpmodeup = taglist->data;
+				config_set_keymod (SECTION, "TPModeUpKey", base->keytpmodeup, taglist->data);
+			} else
+				taglist->data = base->modtpmodeup;
 			break;
 		case TAG_TPMODEDOWNKEY:
-			if (cfgure)	base->keytpmodedn = taglist->data;
-			else		taglist->data = base->keytpmodedn;
+			if (cfgure)	{
+				base->keytpmodedn = taglist->data;
+				config_set_keymod (SECTION, "TPModeDownKey", taglist->data, base->modtpmodedn);
+			} else
+				taglist->data = base->keytpmodedn;
 			break;
 		case TAG_TPMODEDOWNMOD:
-			if (cfgure)	base->modtpmodedn = taglist->data;
-			else		taglist->data = base->modtpmodedn;
+			if (cfgure)	{
+				base->modtpmodedn = taglist->data;
+				config_set_keymod (SECTION, "TPModeDownKey", base->keytpmodedn, taglist->data);
+			} else
+				taglist->data = base->modtpmodedn;
 			break;
 		case TAG_TPMODE:
 			if (cfgure)	{
 				base->trackpad_mode = taglist->data > TRACKPAD_LAST ? 0 : taglist->data;
 				trackpad_set_config(base->trackpad_mode);
+				config_set_option (SECTION, "TPMode", TPModeList, base->trackpad_mode);
 				singletag_to_clients (CHANGEVALUE, TAG_TPMODE, base->trackpad_mode);
 			} else
 				taglist->data = base->trackpad_mode;
 			break;
 		case TAG_NOTAPTYPING:
-			if (cfgure)	base->flags.notap_typing = taglist->data & 1;
-			else		taglist->data = base->flags.notap_typing;
+			if (cfgure)	{
+				base->flags.notap_typing = taglist->data & 1;
+				config_set_boolean (SECTION, "NoTapTyping",	base->flags.notap_typing);
+			} else
+				taglist->data = base->flags.notap_typing;
 			break;
 		case TAG_KBDMODE:
 			if (cfgure) {
@@ -455,6 +440,7 @@ pmac_handle_tags (int cfgure, struct tagitem *taglist)
 						print_msg (PBB_WARN, _("ADB keyboard can't disable fnmode, force mode to 'fkeyslast'.\n"));
 						base->keyboard_mode = KEYBOARD_FNBACK;
 					}
+					config_set_option (SECTION, "KBDMode", KBDModeList, base->keyboard_mode);
 					base->flags.set_kbdmode = 1;     /* request a secure function call */
 				}
 			} else
@@ -465,6 +451,7 @@ pmac_handle_tags (int cfgure, struct tagitem *taglist)
 				batlog_save();
 				base->batlog_mode = taglist->data > BATLOG_LAST ? 0 : taglist->data;
 				base->batlog_cycle = batlog_setup();		
+				config_set_option (SECTION, "Batlog", BatlogModeList, base->batlog_mode);
 			} else
 				taglist->data = base->batlog_mode;
 			break;
@@ -477,7 +464,7 @@ pmac_handle_tags (int cfgure, struct tagitem *taglist)
 			break;
 		case TAG_IDENTITY:
 			if (cfgure)	tagerror (taglist, E_NOWRITE);
-			else		taglist->data = (long) base->identity;
+			else		taglist->data = (long) base->identity->str;
 			break;
 		case TAG_SLEEPSUPPORTED:
 			if (cfgure)	tagerror (taglist, E_NOWRITE);
@@ -509,13 +496,17 @@ pmac_handle_tags (int cfgure, struct tagitem *taglist)
 			if (!haveI2CAmbient() && !havePMUAmbient())
 				tagerror (taglist, E_NOSUPPORT);
 			else if (cfgure)
-#if defined(DEBUG) && SIMUAMBIENT
+#if defined(DEBUG) && SIMU_AMBIENT
 				base->ambient = taglist->data;
 #else
 				tagerror (taglist, E_NOWRITE);
 #endif
 			else
-				taglist->data = getAmbient();
+				taglist->data = getRawAmbient();
+			break;
+		case TAG_AMBIENTLIGHTMAX:
+			if (cfgure)	tagerror (taglist, E_NOWRITE);
+			else		taglist->data = haveI2CAmbient() ? 1600 : 2048;
 			break;
 		case TAG_KEYBLIGHTLEVEL:   /* private tag */
 			if (cfgure)
@@ -528,7 +519,8 @@ pmac_handle_tags (int cfgure, struct tagitem *taglist)
 		case TAG_SYSINFOSLAVE:
 			if (cfgure) ;
 			else {
-				taglist->data |= haveADBKeyboard () ? 0 : SYSINFO_PB_USB_KBD;
+				if (base->version != OHARE_PMU)
+					taglist->data |= haveADBKeyboard () ? 0 : SYSINFO_PB_USB_KBD;
 				taglist->data |= haveADBTrackpad () ? 0 : SYSINFO_PB_USB_TPAD;
 			}
 		}
@@ -693,8 +685,7 @@ activate_sleepmode ()
 #endif
 
 	tpmode = trackpad_get_config ();
-	if (base->version != OHARE_PMU)
-		kbdmode = keyboard_get_config ();
+	kbdmode = keyboard_get_config ();
 	sync();
 #ifndef WITH_PMUD
 	/* PMU_IOC_SLEEP only succeeds if called as root.
@@ -720,8 +711,7 @@ activate_sleepmode ()
 		tcp_close(fd);
 	}
 #endif
-	if (base->version != OHARE_PMU)
-		keyboard_set_config(kbdmode);
+	keyboard_set_config(kbdmode);
 }
 
 /* This function gets the battery status from the PMU kernel driver via
@@ -861,29 +851,6 @@ pmac_update_flags()
 	return (envold ^ envnew);
 }
 
-void
-pmac_set_lcdbacklight (int val)
-{
-	struct moddata_pmac *base = &modbase_pmac;
-	int ambient;
-	
-	if (val == -1)   /* no backlight controller available */
-		return;
-	
-	ambient = getRawAmbient () - base->lcdfeedback;
-	process_queue_single (CONFIGQUEUE, TAG_HW_BACKLIGHT, val);
-	base->lcdfeedback = val ? getRawAmbient () - ambient : 0;
-
-#if defined(DEBUG) && (TESTAMBIENT || SIMUAMBIENT)
-	ambient = getRawAmbient ();
-	printf("LCD Level: %2d, Raw (%4d) - LCD Feedback (%4d) = Ambient (%4d or %2d%%)\n",
-		val, ambient, base->lcdfeedback, ambient - base->lcdfeedback,
-		getAmbient());
-	fflush(stdout);
-#endif
-	return;
-}
-
 /* ----------------------------  GENERIC FUNCTIONS ----------------------------- */
 int
 isModifier (int code, int mod)
@@ -908,12 +875,32 @@ isModifier (int code, int mod)
 	return 0;  /* key is not a modifier */
 }
 
+/**
+ * @brief  Checks if the PowerBook has an ADB keyboard
+ *
+ * This function uses the machine id to determine if the PowerBook
+ * has an ABD keyboard or not.
+ *
+ * Very early PowerBooks doesn't have a machine code. On those
+ * machines a pseude machine code is calculated from the PMU
+ * version. PMU version 9 results in a machine code of 01, 10 in
+ * 02 and 11 in 03.
+ *
+ * PowerBooks with OHARE PMU (version 9) were reported not to support
+ * keyboard configuration. Despite of having an ADB keyboard, this
+ * function reports FALSE for OHARE based PowerBooks. Pbbuttonsd will
+ * think a OHARE based PowerBook has an USB keyboard, but this has no
+ * consequences because the appropriate file in /sys are not availabe.
+ *
+ * @return  TRUE, if the PowerBook has an ADB keyboard or
+ *          FALSE otherwise.
+ */
 int
 haveADBKeyboard ()
 {
 	struct moddata_pmac *base = &modbase_pmac;
 	
-	if (base->machine > 0) {
+	if (base->machine > 1) {
 		if (base->machine < 0x56)              /*  0 - 56 -> true  */
 			return 1;
 		if (base->machine > 0x67)              /* 68 - ?? -> false */
@@ -928,13 +915,27 @@ haveADBKeyboard ()
 	return 0;
 }
 
+/**
+ * @brief  Checks if the PowerBook has an ADB Trackpad
+ *
+ * This function uses the machine id to determine if the PowerBook
+ * has an ABD trackpad or not.
+ *
+ * Very early PowerBooks doesn't have a machine code. On those
+ * machines a pseude machine code is calculated from the PMU
+ * version. PMU version 9 results in a machine code of 01, 10 in
+ * 02 and 11 in 03.
+ *
+ * @return  TRUE, if the PowerBook has an ADB trackpad or
+ *          FALSE otherwise.
+ */
 int
 haveADBTrackpad ()
 {
 	struct moddata_pmac *base = &modbase_pmac;
 	
 	if (base->machine > 0) {
-		if (base->machine < 0x56)              /*  0 - 55 -> true  */
+		if (base->machine < 0x56)              /*  1 - 55 -> true  */
 			return 1;
 		if (base->machine > 0x67)              /* 68 - ?? -> false */
 			return 0;
@@ -952,12 +953,12 @@ int
 haveI2CAmbient ()
 {
 	struct moddata_pmac *base = &modbase_pmac;
-#if defined(DEBUG) && SIMUAMBIENT
+#if defined(DEBUG) && SIMU_AMBIENT
 	return 1;
 #endif
 	
 	if ((base->machine >= 0x51) && (base->machine <= 0x57))
-		if (base->lmuaddr != 0 && strlen(base->i2cdev) > 0)   
+		if (base->lmuaddr != 0 && base->i2cdev->len > 0)   
 			return 1;
 	return 0;
 }
@@ -987,16 +988,22 @@ keyboard_get_config ()
 				config = ADBBuffer[3] & 0x01 ? KEYBOARD_FNTOP : KEYBOARD_FNBACK;
 			}
 		}
-	} else if ((fd = open (PATH_FNMODE, O_RDONLY)) >= 0) {
-		if ((n = read (fd, ADBBuffer, ADB_BUFSIZE-1)) > 0) {
-			ADBBuffer[n] = 0; /* terminate read buffer */
-			config = atoi((const char*)ADBBuffer);
+	} else {
+		if ((fd = open ("/sys/module/usbhid/parameters/pb_fnmode", O_RDONLY)) < 0)
+			fd = open ("/sys/module/hid/parameters/pb_fnmode", O_RDONLY);
+		
+		if (fd >= 0) {
+			if ((n = read (fd, ADBBuffer, ADB_BUFSIZE-1)) > 0) {
+				ADBBuffer[n] = 0; /* terminate read buffer */
+				config = atoi((const char*)ADBBuffer);
 #ifdef DEBUG
-			printf ("DBG: Keyboard config read: '%s' -> %d.\n", ADBBuffer, config);			
+				printf ("DBG: Keyboard config read: '%s' -> %d.\n", ADBBuffer, config);			
 #endif
+			}
+			close (fd);
 		}
-		close (fd);
 	}
+
 	return config;
 }
 
@@ -1016,14 +1023,20 @@ keyboard_set_config (int config)
 				adb_write_reg (base->fd_adb, ADBBuffer, n, dev, 1);
 			}
 		}
-	} else if ((fd = open (PATH_FNMODE, O_WRONLY)) >= 0) {
-		sprintf ((char*)ADBBuffer, "%d", config);
-		write (fd, ADBBuffer, strlen((const char*)ADBBuffer));
+	} else {
+		if ((fd = open ("/sys/module/usbhid/parameters/pb_fnmode", O_WRONLY)) < 0)
+			fd = open ("/sys/module/hid/parameters/pb_fnmode", O_WRONLY);
+		
+		if (fd >= 0) {
+			sprintf ((char*)ADBBuffer, "%d", config);
+			write (fd, ADBBuffer, strlen((const char*)ADBBuffer));
 #ifdef DEBUG
-		printf ("DBG: Keyboard config written: '%s' -> %d.\n", ADBBuffer, config);			
+			printf ("DBG: Keyboard config written: '%s' -> %d.\n", ADBBuffer, config);			
 #endif
-		close (fd);
+			close (fd);
+		}
 	}
+
 }
 
 int
@@ -1435,30 +1448,32 @@ findLMUDevice(int addr)
 	return buffer;
 }
 
-/* This function returns the a compensated ambient light sensor
- * value in percent. The reduce flickering through shadows or
- * spot lights at one of the sensors the sensor with the highest
- * value is choosen.
- */
-int
-getAmbient ()
-{
-	struct moddata_pmac *base = &modbase_pmac;
-	int rc, ambientmax;
-
-	ambientmax = haveI2CAmbient() ? 1600 : 2048;
-	rc = (int) (((getRawAmbient() - base->lcdfeedback) 
-				* 100) / ambientmax);
-
-	if (rc < 0)	  rc = 0;
-	if (rc > 100) rc = 100;
-	return rc;
-}
-
-/* The first ambient light sensor was connected to the I2C
+/**
+ * @brief Get the raw ambient light sensor value from hardware
+ *
+ * This function read the ambient sensor from hardware. Two
+ * interfaces are supported:
+ *   @li  I2C
+ *   @li  PMU
+ *
+ * The first ambient light sensor was connected to the I2C
  * Bus but recent PowerBooks 5.8 and 5.9 have a new ambient
  * light sensor connected to the PMU. This functions reads
- * the current sensor value from the right interface.
+ * the current sensor value from the correct interface.
+ *
+ * The returned sensor values are checked for boundaries and if
+ * any value is suspect the fuction return -1. In this case the
+ * suspect is guilty until its innocence is proven.
+ *
+ * The PowerBooks have two independent ambient light sensors.
+ * To reduce flickering through shadows or spot lights at one of
+ * the sensors the sensor with the highest value is returned.
+ *
+ * The maximum ambient brightness depends on the used sensor.
+ * It is 1600 for the I2C sensor and 2048 for the PMU sensor.
+ *
+ * @return  The raw ambient light value or -1 if something went
+ *          wrong and the value is invalid.
  */
 int
 getRawAmbient ()
@@ -1467,7 +1482,7 @@ getRawAmbient ()
 	unsigned char buf[ADB_BUFSIZE];
 	int fd, rc = -1, rc2;
 	
-#if defined(DEBUG) && SIMUAMBIENT
+#if defined(DEBUG) && SIMU_AMBIENT
 	int ambient, level, idx, max;
 
 	/* this array contains sensor values depending on the
@@ -1479,10 +1494,43 @@ getRawAmbient ()
 	 * will also work with more than 16 LCD brightness
 	 * levels.
 	 */
-	int asens[] = {2, 182, 217, 248, 309, 337, 381, 423, 461,
-		           499, 538, 576, 620, 1072, 1073, 1077};
-		       
-	level = process_queue_single (QUERYQUEUE, TAG_HW_BACKLIGHT, -1);
+	int asens0[] ={  2, 182, 217, 248, 309, 337, 381, 423,
+		           461, 499, 538, 576, 620,1072,1073,1077 };
+		      
+	int asens[] = {  0,   2,  20,  50,  80,  90, 100, 110,
+	               120, 130, 140, 150, 160, 170, 175, 182,
+	               184, 187, 189, 192, 194, 196, 199, 201,
+	               203, 206, 208, 211, 213, 215, 218, 220,
+	               222, 225, 227, 230, 232, 234, 237, 239,
+	               241, 244, 246, 249, 251, 253, 256, 258,
+	               260, 263, 265, 268, 270, 272, 275, 277,
+                   279, 282, 284, 287, 289, 291, 294, 296,
+	               298, 301, 303, 305, 308, 310, 313, 315,
+	               317, 320, 322, 324, 327, 329, 332, 334,
+	               336, 339, 341, 343, 346, 348, 351, 353,
+	               355, 358, 360, 362, 365, 367, 370, 372,
+	               374, 377, 379, 381, 384, 386, 389, 391,
+	               393, 396, 398, 400, 403, 405, 408, 410,
+	               412, 415, 417, 419, 422, 424, 427, 429,
+	               431, 434, 436, 438, 441, 443, 445, 448,
+	               450, 453, 455, 457, 460, 462, 464, 467,
+	               469, 472, 474, 476, 479, 481, 483, 486,
+	               488, 491, 493, 495, 498, 500, 502, 505,
+	               507, 510, 512, 514, 517, 519, 521, 524,
+	               526, 529, 531, 533, 536, 538, 540, 543,
+	               545, 548, 550, 552, 555, 557, 559, 562,
+	               564, 567, 569, 571, 574, 576, 578, 581,
+	               583, 586, 588, 590, 593, 595, 597, 600,
+	               602, 604, 607, 609, 612, 614, 623, 673,
+	               723, 773, 823, 873, 923, 973,1023,1073,
+	              1073,1073,1073,1073,1073,1073,1073,1073,
+	              1073,1073,1073,1073,1073,1073,1073,1074,
+	              1074,1074,1074,1074,1074,1074,1074,1074,
+	              1074,1074,1075,1075,1075,1076,1076,1076,
+	              1076,1076,1077,1077,1077,1077,1077,1077,
+	              1077,1077,1077,1077,1077,1077,1077,1077 };
+
+	level = process_queue_single (QUERYQUEUE, TAG_BACKLIGHTLEVEL, -1);
 	max   = process_queue_single (QUERYQUEUE, TAG_BACKLIGHTMAX, -1);
 	idx = level * sizeof(asens) / sizeof(int) / (max + 1);
 	ambient = asens[level == -1 ? 0 : idx] + base->ambient;
@@ -1490,7 +1538,7 @@ getRawAmbient ()
 	return ambient;
 #else	
 	if (haveI2CAmbient()) {
-		if ((fd = open (base->i2cdev, O_RDWR)) >= 0 ) {
+		if ((fd = open (base->i2cdev->str, O_RDWR)) >= 0 ) {
 			if (ioctl (fd, I2C_SLAVE, base->lmuaddr) >= 0 )
 				if (read (fd, buf, 4 ) == 4) {
 					rc  = (buf[0] << 8) | buf[1];
@@ -1515,7 +1563,7 @@ getRawAmbient ()
 void
 setKBDIllumination (unsigned short level)
 {
-#if !(defined(DEBUG) && SIMUAMBIENT)
+#if !(defined(DEBUG) && SIMU_AMBIENT)
 	struct moddata_pmac *base = &modbase_pmac;
 	unsigned char buf[ADB_BUFSIZE];
 	int fd;
@@ -1537,7 +1585,7 @@ setKBDIllumination (unsigned short level)
 		buf[1] = (unsigned char) level >> 4;
 		buf[2] = (unsigned char) (level & 0x0F) << 4;
 
-		if ((fd = open (base->i2cdev, O_RDWR)) >= 0 ) {
+		if ((fd = open (base->i2cdev->str, O_RDWR)) >= 0 ) {
 			if (ioctl (fd, I2C_SLAVE, base->lmuaddr) >= 0 )
 				write (fd, buf, 3);
 			close (fd);
@@ -1548,9 +1596,8 @@ setKBDIllumination (unsigned short level)
 		send_pmu_request (base->fd_adb, buf, 4, 0x4F, 0, 0, level);
 	}
 #else
-	printf("KBD Level: %2d at %2d%% Ambient\n",
-		level, getAmbient());
-	fflush(stdout);
+	printf("KBD Level: %2d at %3d%% Ambient\n",
+		level, getRawAmbient() * 100 / 1600);
 #endif
 }
 
@@ -1731,4 +1778,3 @@ batlog_write(long timerem)
 		}
 	}
 }
-#endif /* WITH_MODULE_PMAC */
