@@ -1,14 +1,18 @@
-/* --------------------------------------------------------------------------
- * module_powersave.c
- * code for powermanagement module
+/**
+ * @brief Powermanagement module
  *
- * Copyright 2002 Matthias Grimm
+ * This module contains the powermanagement logic. It connects multiple
+ * sources and uses this information to serve the user best regarding 
+ * battery monitoring, power saving, data security and much more.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version
- * 2 of the License, or (at your option) any later version.
- * -------------------------------------------------------------------------*/
+ * version 2 as published by the Free Software Foundation
+ * (http://www.gnu.org/licenses/gpl.html)
+ *
+ * @file    src/module_powersave.c
+ * @author  Matthias Grimm <matthias.grimm@users.sourceforge.net>
+ */
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
@@ -35,16 +39,19 @@
 
 #include "gettext_macros.h"
 #include "input_manager.h"
-#include "config_manager.h"
 #include "module_powersave.h"
 #include "module_display.h"
 #include "support.h"
 #include "debug.h"
 
+#include "class_config.h"
+
 #ifdef WITH_IBAM
 #  include "ibam_stub.h"
 #endif
 
+#define SECTION "MODULE POWERSAVE"
+
 /* --- private module data structure of module powermanagement --- */
 struct moddata_power {
 	struct modflags_power flags;
@@ -79,6 +86,10 @@ struct moddata_power {
 #endif
 } modbase_power;
 
+static const char *PolicyList[] = { "nochange", "powersave", "custom", "performance", NULL };
+static const char *ActionList[] = { "none", "suspend-to-ram", "suspend-to-disk", "blankscreen", "shutdown", NULL };
+static const char *EmergencyList[] = { "sleep", "signal", "command", NULL };
+
 /* --- interface functions of module powermanagement --- */
 
 /* Init function called by the program init procedure. Function is known by
@@ -89,177 +100,82 @@ int
 power_init ()
 {
 	struct moddata_power *base = &modbase_power;
-	static char scriptbuffer1[STDBUFFERLEN];
-	static char scriptbuffer2[STDBUFFERLEN];
-#ifdef WITH_IBAM
-	static char scriptbuffer3[STDBUFFERLEN];
 	int err;
-#endif
-	int sid;
 
-	bzero (base, sizeof (struct moddata_power));
-	bzero (scriptbuffer1, STDBUFFERLEN);  /* no default for script_pmcs */
-	base->script_pmcs  = scriptbuffer1;
-	sprintf (scriptbuffer2, DEFAULT_NETLOAD_DEV);
-	base->netload_dev  = scriptbuffer2;
-#ifdef WITH_IBAM
-	base->ibam_datadir  = scriptbuffer3;
-	if ((err = copy_path (DEFAULT_IBAM_DATADIR, base->ibam_datadir, TYPE_DIR, CPFLG_PBBONLY))) 
-		return err;
-#endif
-	base->mode         = MODE_AWAKE;
-	base->policy       = POLICY_PERFORMANCE;
-	
-	if ((sid = registerCfgSection ("MODULE POWERSAVE")) == 0) {
-		print_msg (PBB_ERR, _("Can't register configuration section %s, out of memory."), "MODULE POWERSAVE");
-		return E_NOMEM;
-	} else {
-		registerCfgOptionList (sid, "onAC_Policy", TAG_ONAC_POLICY, 0,
-				N_("'"POLICY_NONE_NAME"', '"POLICY_POWERSAVE_NAME"', '"POLICY_USER_NAME"' or '"POLICY_PERFORMANCE_NAME"'"),
-				POLICY_NONE_NAME, POLICY_POWERSAVE_NAME, POLICY_USER_NAME, POLICY_PERFORMANCE_NAME, NULL);
-		registerCfgOptionList (sid, "onAC_TimerAction", TAG_ONAC_TIMERACTION, 0,
-				N_("'"ACTION_NONE_NAME"', '"ACTION_TORAM_NAME"', '"ACTION_TODISK_NAME"', '"ACTION_BLANK_NAME"' or '"ACTION_SHUTDOWN_NAME"'"),
-				ACTION_NONE_NAME, ACTION_TORAM_NAME, ACTION_TODISK_NAME, ACTION_BLANK_NAME, ACTION_SHUTDOWN_NAME, NULL);
-		registerCfgOptionList (sid, "onAC_CoverAction",  TAG_ONAC_COVERACTION, 0,
-				N_("see TimerAction for possible values"),
-				ACTION_NONE_NAME, ACTION_TORAM_NAME, ACTION_TODISK_NAME, ACTION_BLANK_NAME, ACTION_SHUTDOWN_NAME, NULL);
-		registerCfgOptionList (sid, "onAC_KeyAction", TAG_ONAC_KEYACTION, 0,
-				N_("Action (see TimerAction) for the sleepkey"),
-				ACTION_NONE_NAME, ACTION_TORAM_NAME, ACTION_TODISK_NAME, ACTION_BLANK_NAME, ACTION_SHUTDOWN_NAME, NULL);
-		registerCfgOptionInt (sid, "onAC_SuspendTime", TAG_ONAC_TIMESUSPEND, 0,
-				NULL);
-		registerCfgOptionInt (sid, "onAC_DimTime",     TAG_ONAC_TIMEDIM,     0,
-				NULL);
-
-		registerCfgOptionList (sid, "onBattery_Policy", TAG_ONBATT_POLICY, 0,
-				N_("'"POLICY_NONE_NAME"', '"POLICY_POWERSAVE_NAME"', '"POLICY_USER_NAME"' or '"POLICY_PERFORMANCE_NAME"'"),
-				POLICY_NONE_NAME, POLICY_POWERSAVE_NAME, POLICY_USER_NAME, POLICY_PERFORMANCE_NAME, NULL);
-		registerCfgOptionList (sid, "onBattery_TimerAction", TAG_ONBATT_TIMERACTION, 0,
-				N_("'"ACTION_NONE_NAME"', '"ACTION_TORAM_NAME"', '"ACTION_TODISK_NAME"', '"ACTION_BLANK_NAME"' or '"ACTION_SHUTDOWN_NAME"'"),
-				ACTION_NONE_NAME, ACTION_TORAM_NAME, ACTION_TODISK_NAME, ACTION_BLANK_NAME, ACTION_SHUTDOWN_NAME, NULL);
-		registerCfgOptionList (sid, "onBattery_CoverAction",  TAG_ONBATT_COVERACTION, 0,
-				N_("see TimerAction for possible values"),
-				ACTION_NONE_NAME, ACTION_TORAM_NAME, ACTION_TODISK_NAME, ACTION_BLANK_NAME, ACTION_SHUTDOWN_NAME, NULL);
-		registerCfgOptionList (sid, "onBattery_KeyAction", TAG_ONBATT_KEYACTION, 0,
-				N_("Action (see TimerAction) for the sleepkey"),
-				ACTION_NONE_NAME, ACTION_TORAM_NAME, ACTION_TODISK_NAME, ACTION_BLANK_NAME, ACTION_SHUTDOWN_NAME, NULL);
-		registerCfgOptionInt (sid, "onBattery_SuspendTime", TAG_ONBATT_TIMESUSPEND, 0,
-				NULL);
-		registerCfgOptionInt (sid, "onBattery_DimTime",     TAG_ONBATT_TIMEDIM,     0,
-				NULL);
-		
-		registerCfgOptionKey (sid, "SleepKey", TAG_SLEEPKEY, TAG_SLEEPMOD, 0,
-				NULL);
-		registerCfgOptionInt (sid, "SleepKeyDelay", TAG_SLEEPKEYDELAY, 0,
-				N_("values > 0 may be dangerous, if the power key is used to trigger sleep"));
-		registerCfgOptionInt (sid, "BWL_first", TAG_BWLFIRST, 0,
-				N_("first battery warnlevel, time in minutes"));
-		registerCfgOptionInt (sid, "BWL_second", TAG_BWLSECOND, 0,
-				N_("second battery warnlevel, time in minutes"));
-		registerCfgOptionInt (sid, "BWL_last", TAG_BWLLAST, 0,
-				N_("last battery warnlevel, time in minutes"));
-		registerCfgOptionString (sid, "Script_PMCS", TAG_SCRIPTPMCS, 0,
-				NULL);
-		registerCfgOptionList (sid, "EmergencyAction", TAG_EMERGENCYACTION, 0,
-				N_("action, if battery is critically low"),
-				EMA_SLEEP_NAME, EMA_SIGNAL_NAME, EMA_COMMAND_NAME, NULL);
-		registerCfgOptionBool (sid, "HeartbeatBeep", TAG_HEARTBEATBEEP, 0,
-				N_("beep, if nothing else showed that the computer lives"));
-		registerCfgOptionBool (sid, "CPULoad_sleeplock", TAG_CPULOADSLEEPLOCK, 0,
-				NULL);
-		registerCfgOptionInt (sid, "CPULoad_min", TAG_CPULOADMIN, 0,
-				N_("value in percent"));
-		registerCfgOptionInt (sid, "CPULoad_period", TAG_CPULOADPERIOD, 0,
-				N_("time in seconds"));
-		registerCfgOptionBool (sid, "NETLoad_sleeplock", TAG_NETLOADSLEEPLOCK, 0,
-				NULL);
-		registerCfgOptionInt (sid, "NETLoad_min", TAG_NETLOADMIN, 0,
-				N_("traffic in Bytes/s"));
-		registerCfgOptionInt (sid, "NETLoad_period", TAG_NETLOADPERIOD, 0,
-				N_("time in seconds"));
-		registerCfgOptionString (sid, "NETLoad_device", TAG_NETLOADDEV, 0,
-				NULL);
-#ifdef WITH_IBAM
-		registerCfgOptionString (sid, "IBAM_DataDir", TAG_IBAMDATADIR, 0,
-				NULL);
-#endif
-	}
-
-	ipc_protect_tag (TAG_SCRIPTPMCS);  /* tags only for privileged use */
-	ipc_protect_tag (TAG_IBAMDATADIR);
-
-	register_function (QUERYQUEUE, power_query);
-	register_function (CONFIGQUEUE, power_configure);
-	return 0;
-}
-
-int
-power_open (struct tagitem *taglist)
-{
-	struct moddata_power *base = &modbase_power;
-	
 	power_sync(); /* synchronise some data with module_pmac */
-	
-	/* default profile for running on AC power */
-	base->onAC.policy	        = POLICY_PERFORMANCE;
-	base->onAC.timeraction      = base->flags.sleep_supported ? ACTION_TORAM : ACTION_BLANK;
-	base->onAC.coveraction      = base->flags.sleep_supported ? ACTION_TORAM : ACTION_BLANK;
-	base->onAC.keyaction        = base->flags.sleep_supported ? ACTION_TORAM : ACTION_BLANK;
-	base->onAC.suspendtime      = AC_TIME_SLEEP;
-	base->onAC.dimtime          = AC_TIME_DIM;
+
+	base->mode                = MODE_AWAKE;
+	base->policy              = POLICY_PERFORMANCE;
+	base->sleepcountdown      = SLEEPCOUNTDOWN; /* counter for last 5 seconds before sleep */
+	base->sleeptime           = 0;  /* counter for time till sleep */
+	base->warnlevel           = 0;  /* current battery warnlevel */
+	base->flags.sigpwr_sent   = 0;  /* currently no sigpwr in progress */
+	base->flags.sleeplock_cpu = 0;  /* cpu sleeplock open */
+	base->flags.sleeplock_net = 0;  /* net sleeplock open */
+	base->flags.heartbeat     = 0;  /* heartbeat beep disabled */
+	base->flags.coveropen     = 1;  /* cover open by default same as in laptop module */
+
+	/* profile for running on AC power */
+	base->onAC.policy	   = config_get_option (SECTION, "onAC_Policy", PolicyList, POLICY_PERFORMANCE);
+	base->onAC.timeraction = config_get_option (SECTION, "onAC_TimerAction", ActionList, ACTION_TORAM);
+	base->onAC.coveraction = config_get_option (SECTION, "onAC_CoverAction", ActionList, ACTION_TORAM);
+	base->onAC.keyaction   = config_get_option (SECTION, "onAC_KeyAction", ActionList, ACTION_TORAM);
+	base->onAC.suspendtime = config_get_int    (SECTION, "onAC_SuspendTime", AC_TIME_SLEEP);
+	base->onAC.dimtime     = config_get_int    (SECTION, "onAC_DimTime", AC_TIME_DIM);
  	
-	/* default profile for running on battery */
-	base->onBattery.policy	    = POLICY_POWERSAVE;
-	base->onBattery.timeraction = base->flags.sleep_supported ? ACTION_TORAM : ACTION_BLANK;
-	base->onBattery.coveraction = base->flags.sleep_supported ? ACTION_TORAM : ACTION_BLANK;
-	base->onBattery.keyaction   = base->flags.sleep_supported ? ACTION_TORAM : ACTION_BLANK;
-	base->onBattery.suspendtime = BATTERY_TIME_SLEEP;
-	base->onBattery.dimtime     = BATTERY_TIME_DIM;
+	/* profile for running on battery */
+	base->onBattery.policy	    = config_get_option (SECTION, "onBattery_Policy", PolicyList, POLICY_POWERSAVE);
+	base->onBattery.timeraction = config_get_option (SECTION, "onBattery_TimerAction", ActionList, ACTION_TORAM);
+	base->onBattery.coveraction = config_get_option (SECTION, "onBattery_CoverAction", ActionList, ACTION_TORAM);
+	base->onBattery.keyaction   = config_get_option (SECTION, "onBattery_KeyAction", ActionList, ACTION_TORAM);
+	base->onBattery.suspendtime = config_get_int    (SECTION, "onBattery_SuspendTime", BATTERY_TIME_SLEEP);
+	base->onBattery.dimtime     = config_get_int    (SECTION, "onBattery_DimTime", BATTERY_TIME_DIM);
 
-	/* definitions for immediatly sleep function */
-	base->keysleep              = KEY_POWER;
-	base->modsleep              = MOD_NONE;
-	base->sleepkeydelaytime     = 0;       /* time delay till reaction */
-	base->sleeptime             = 0;      /* counter for time till sleep */
-	base->sleepcountdown        = 60;    /* counter for last 5 seconds before sleep */
+	base->keysleep  = config_get_keycode (SECTION, "SleepKey", KEY_POWER);
+	base->modsleep  = config_get_modmask (SECTION, "SleepKey", MOD_NONE);
+	base->BWL1      = config_get_int    (SECTION, "BWL_First", BAT_WARNLEVEL1);
+	base->BWL2      = config_get_int    (SECTION, "BWL_Second", BAT_WARNLEVEL2);
+	base->BWL3      = config_get_int    (SECTION, "BWL_Last", BAT_WARNLEVEL3);
+	base->emergency = config_get_option (SECTION, "EmergencyAction", EmergencyList, EMA_SLEEP);
 
-	/* definitions for battery supervising functions */
-	base->BWL1                  = BAT_WARNLEVEL1;
-	base->BWL2                  = BAT_WARNLEVEL2;
-	base->BWL3                  = BAT_WARNLEVEL3;
-	base->warnlevel             = 0;          /* current battery warnlevel */
-	base->emergency             = base->flags.sleep_supported ? EMA_SLEEP : EMA_SIGNAL; /* default emergency action */
-	base->flags.sigpwr_sent     = 0;  /* currently no sigpwr in progress */
+	base->sleepkeydelaytime     = config_get_int     (SECTION, "SleepKeyDelay", 0);
+	base->flags.cpuload_enable  = config_get_boolean (SECTION, "CPULoad_sleeplock", FALSE);
+	base->cpuload_min           = config_get_percent (SECTION, "CPULoad_min", 20);
+	base->cpuload_period        = config_get_int     (SECTION, "CPULoad_period", 60);
+	base->flags.netload_enable  = config_get_boolean (SECTION, "NETLoad_sleeplock", FALSE);
+	base->netload_min           = config_get_int     (SECTION, "NETLoad_min", 4096);
+	base->netload_period        = config_get_int     (SECTION, "NetLoad_period", 60);
+	base->flags.heartbeat_enable= config_get_boolean (SECTION, "HeartbeatBeep", TRUE);
+	base->script_pmcs           = config_get_string  (SECTION, "Script_PMCS", "");
+	base->netload_dev           = config_get_string  (SECTION, "NETLoad_device", DEFAULT_NETLOAD_DEV);
 
-	/* definitions for sleeplocks */
-	base->flags.cpuload_enable  = 0;    /* disabled by default */
-	base->flags.sleeplock_cpu   = 0;    /* lock open */
-	base->cpuload_min           = 20;   /* lock closed if cpuload <20% */
-	base->cpuload_period        = 60;   /*  for >60s, if enabled */
-	base->flags.netload_enable  = 0;    /* disabled by default */
-	base->flags.sleeplock_net   = 0;    /* lock open */
-	base->netload_min           = 4096; /* lock closed if netload <4 KB/s */
-	base->netload_period        = 60;   /*  for >60s, if enabled  */
+	/* HACK: The pmcs script sould be started at startup but it will only
+	 * start if the status has changed (policy or powersource). We set an
+	 * impossible value here and pbbuttonsd.c will send a TAG_POWERCHANGE
+	 * to correct it and start the script. Not elegant but works */
+	base->powersource = 2;
 
-	base->flags.heartbeat       = 0;  /* heartbeat beep disabled */
-	base->flags.heartbeat_enable= 1;  /* heartbeat beep is possible by default */ 
-	base->flags.coveropen       = 1;  /* cover open by default same as in laptop module */
-
-	base->powersource = 2; /* HACK: The pmcs script sould be started at startup but
-				  it will only start if the status has changed (policy
-				  or powersource). We set an impossible value here and pbbuttonsd.c
-				  will send a TAG_POWERCHANGE to correct it and start the script
-				  Not elegant but works */
-	
 #ifdef WITH_IBAM
+	base->ibam_datadir          = config_get_string  (SECTION, "IBAM_DataDir", DEFAULT_IBAM_DATADIR);
+
 	base->timeremaining = -1; /* will be updated from timer1000 */
 	base->chargetime    = -1;
+
+	if ((err = check_path (base->ibam_datadir, TYPE_DIR, CPFLG_PBBONLY))) 
+		return err;
+
 	if (!ibam_init(base->ibam_datadir)) {  /* create ibam object */
 		print_msg (PBB_ERR, _("Can't create IBaM object, out of memory.\n"));
 		return E_NOMEM;
 	}
+
+	ipc_protect_tag (TAG_IBAMDATADIR);
 #endif
-		
+	
+	ipc_protect_tag (TAG_SCRIPTPMCS);  /* tags only for privileged use */
+
+	register_function (QUERYQUEUE, power_query);
+	register_function (CONFIGQUEUE, power_configure);
 	register_function (KBDQUEUE, power_keyboard);
 	register_function (MOUSEQUEUE, power_mouse);
 	register_function (T100QUEUE, power_timer);
@@ -267,19 +183,16 @@ power_open (struct tagitem *taglist)
 	return 0;
 }
 
-int
-power_close ()
-{
-#ifdef WITH_IBAM
-	ibam_exit();
-#endif
-	return 0;
-}
-
-int
+void
 power_exit ()
 {
-	return 0;
+	struct moddata_power *base = &modbase_power;
+#ifdef WITH_IBAM
+	ibam_exit();
+	g_free (base->ibam_datadir);
+#endif
+	g_free (base->script_pmcs);
+	g_free (base->netload_dev);
 }
 
 void
@@ -380,8 +293,12 @@ power_handle_tags (int cfgure, struct tagitem *taglist)
 		case TAG_SCRIPTPMCS:
 			if (cfgure) {
 				if ((check_script ((char *) taglist->data, "sss")) == 0) {
-					if ((err = copy_path ((char *) taglist->data, base->script_pmcs, TYPE_FILE, CPFLG_PBBONLY)))
+					if ((err = check_path ((char *) taglist->data, TYPE_FILE, CPFLG_PBBONLY)))
 						tagerror (taglist, err);
+					else {
+						replace_string (&base->script_pmcs, (char *) taglist->data);
+						config_set_string (SECTION, "Script_PMCS", base->script_pmcs);
+					}
 				} else {
 					print_msg (PBB_ERR, _("Too many formatsigns. Max three %%s allowed in TAG_SCRIPTPMCS.\n"));
 					tagerror (taglist, E_FORMAT);
@@ -392,8 +309,12 @@ power_handle_tags (int cfgure, struct tagitem *taglist)
 #ifdef WITH_IBAM
 		case TAG_IBAMDATADIR:
 			if (cfgure) {
-				if ((err = copy_path ((char *) taglist->data, base->ibam_datadir, TYPE_DIR, CPFLG_PBBONLY)))
+				if ((err = check_path ((char *) taglist->data, TYPE_DIR, CPFLG_PBBONLY)))
 					tagerror (taglist, err);
+				else {
+					replace_string (&base->ibam_datadir, (char *) taglist->data);
+					config_set_string (SECTION, "IBAM_DataDir", base->ibam_datadir);
+				}
 			} else
 				taglist->data = (long) base->ibam_datadir;
 			break;
@@ -402,126 +323,160 @@ power_handle_tags (int cfgure, struct tagitem *taglist)
 		/* Tags for power profiles */	
 		case TAG_ONAC_POLICY:
 			if (cfgure) {
-				if (taglist->data > POLICY_LAST)
-					tagerror(taglist, E_INVALID);
-				else {
+				if (taglist->data <= POLICY_LAST) {
 					base->onAC.policy = taglist->data;
-					if (&base->onAC == base->activeProfile
-					    && !base->flags.usersetpolicy)
+					if (&base->onAC == base->activeProfile && !base->flags.usersetpolicy)
 						power_setpolicy(taglist->data, base->powersource);
-				}
+					config_set_option (SECTION, "onAC_Policy", PolicyList, taglist->data);
+				} else
+					tagerror(taglist, E_INVALID);
 			} else
 				taglist->data = base->onAC.policy;
 			break;
 		case TAG_ONAC_TIMERACTION:
  			if (cfgure) {
-				if (taglist->data > ACTION_LAST)
-					tagerror(taglist, E_INVALID);
-				else
+				if (taglist->data <= ACTION_LAST) {
 					base->onAC.timeraction = taglist->data;
+					config_set_option (SECTION, "onAC_TimerAction", ActionList, taglist->data);
+				} else
+					tagerror(taglist, E_INVALID);
  			} else
 				taglist->data = base->onAC.timeraction;
  			break;
 		case TAG_ONAC_COVERACTION:
 			if (cfgure) {
-				if (taglist->data > ACTION_LAST)
-					tagerror(taglist, E_INVALID);
-				else
+				if (taglist->data <= ACTION_LAST) {
 					base->onAC.coveraction = taglist->data;
+					config_set_option (SECTION, "onAC_CoverAction", ActionList, taglist->data);
+				} else
+					tagerror(taglist, E_INVALID);
 			} else
 				taglist->data = base->onAC.coveraction;
  			break;
 		case TAG_ONAC_KEYACTION:   /* sleepkey */
 			if (cfgure) {
-				if (taglist->data > ACTION_LAST)
-					tagerror(taglist, E_INVALID);
-				else
+				if (taglist->data <= ACTION_LAST) {
 					base->onAC.keyaction = taglist->data;
+					config_set_option (SECTION, "onAC_KeyAction", ActionList, taglist->data);
+				} else
+					tagerror(taglist, E_INVALID);
 			} else
 				taglist->data = base->onAC.keyaction;
 			break;	
 		case TAG_ONAC_TIMESUSPEND:
-			if (cfgure)	base->onAC.suspendtime = taglist->data;
-			else		taglist->data = base->onAC.suspendtime;
+			if (cfgure)	{
+				base->onAC.suspendtime = taglist->data;
+				config_set_int (SECTION, "onAC_SuspendTime", taglist->data);
+			} else
+				taglist->data = base->onAC.suspendtime;
 			break;
 		case TAG_ONAC_TIMEDIM:
-			if (cfgure)	base->onAC.dimtime = taglist->data;
-			else		taglist->data = base->onAC.dimtime;
+			if (cfgure)	{
+				base->onAC.dimtime = taglist->data;
+				config_set_int (SECTION, "onAC_DimTime", taglist->data);
+			} else
+				taglist->data = base->onAC.dimtime;
  			break;
 	
 		case TAG_ONBATT_POLICY:	
 			if (cfgure) {
-				if (taglist->data > POLICY_LAST)
-					tagerror(taglist, E_INVALID);
-				else {
+				if (taglist->data <= POLICY_LAST) {
 					base->onBattery.policy = taglist->data;
-					if (&base->onBattery == base->activeProfile
-					    && !base->flags.usersetpolicy)
+					if (&base->onBattery == base->activeProfile && !base->flags.usersetpolicy)
 						power_setpolicy(taglist->data, base->powersource);
-				}
+					config_set_option (SECTION, "onBattery_Policy", PolicyList, POLICY_POWERSAVE);
+				} else
+					tagerror(taglist, E_INVALID);
 			} else	
 				taglist->data = base->onBattery.policy;
 			break;
 		case TAG_ONBATT_TIMERACTION:
 			if (cfgure) {
-				if (taglist->data > ACTION_LAST)
-					tagerror(taglist, E_INVALID);
-				else
+				if (taglist->data <= ACTION_LAST) {
 					base->onBattery.timeraction = taglist->data;
+					config_set_option (SECTION, "onBattery_TimerAction", ActionList, taglist->data);
+				} else
+					tagerror(taglist, E_INVALID);
 			} else
 				taglist->data = base->onBattery.timeraction;
 			break;
 		case TAG_ONBATT_COVERACTION:
 			if (cfgure) {
-				if (taglist->data > ACTION_LAST)
-					tagerror(taglist, E_INVALID);
-				else
+				if (taglist->data <= ACTION_LAST) {
 					base->onBattery.coveraction = taglist->data;
+					config_set_option (SECTION, "onBattery_CoverAction", ActionList, taglist->data);
+				} else
+					tagerror(taglist, E_INVALID);
 			} else
 				taglist->data = base->onBattery.coveraction;
 			break;
 		case TAG_ONBATT_KEYACTION:   /* sleepkey */
 			if (cfgure) {
-				if (taglist->data > ACTION_LAST)
-					tagerror(taglist, E_INVALID);
-				else
+				if (taglist->data <= ACTION_LAST) {
 					base->onBattery.keyaction = taglist->data;
+					config_set_option (SECTION, "onBattery_KeyAction", ActionList, taglist->data);
+				} else
+					tagerror(taglist, E_INVALID);
 			} else
 				taglist->data = base->onBattery.keyaction;
 			break;
 		case TAG_ONBATT_TIMESUSPEND:
-			if (cfgure)	base->onBattery.suspendtime = taglist->data;
-			else		taglist->data = base->onBattery.suspendtime;
+			if (cfgure)	{
+				base->onBattery.suspendtime = taglist->data;
+				config_set_int (SECTION, "onBattery_SuspendTime", taglist->data);
+			} else
+				taglist->data = base->onBattery.suspendtime;
 			break;
 		case TAG_ONBATT_TIMEDIM:
-			if (cfgure)	base->onBattery.dimtime = taglist->data;
-			else		taglist->data = base->onBattery.dimtime;
+			if (cfgure)	{
+				base->onBattery.dimtime = taglist->data;
+				config_set_int (SECTION, "onBattery_DimTime", taglist->data);
+			} else
+				taglist->data = base->onBattery.dimtime;
 			break;
 		/* Tags for power profiles end */
 
 		case TAG_SLEEPKEY:
-			if (cfgure)	base->keysleep = taglist->data;
-			else		taglist->data = base->keysleep;
+			if (cfgure)	{
+				base->keysleep = taglist->data;
+				config_set_keymod (SECTION, "SleepKey", taglist->data, base->modsleep);
+			} else
+				taglist->data = base->keysleep;
 			break;
 		case TAG_SLEEPMOD:
-			if (cfgure)	base->modsleep = taglist->data;
-			else		taglist->data = base->modsleep;
+			if (cfgure)	{
+				base->modsleep = taglist->data;
+				config_set_keymod (SECTION, "SleepKey", base->keysleep, taglist->data);
+			} else
+				taglist->data = base->modsleep;
 			break;
 		case TAG_SLEEPKEYDELAY:
-			if (cfgure)	base->sleepkeydelaytime = taglist->data;
-			else		taglist->data = base->sleepkeydelaytime;
+			if (cfgure)	{
+				base->sleepkeydelaytime = taglist->data;
+				config_set_int (SECTION, "SleepKeyDelay", taglist->data);
+			} else
+				taglist->data = base->sleepkeydelaytime;
 			break;
 		case TAG_BWLFIRST:
-			if (cfgure)	base->BWL1 = taglist->data;
-			else		taglist->data = base->BWL1;
+			if (cfgure) {
+				base->BWL1 = taglist->data;
+				config_set_int (SECTION, "BWL_First", taglist->data);
+			} else
+				taglist->data = base->BWL1;
 			break;
 		case TAG_BWLSECOND:
-			if (cfgure)	base->BWL2 = taglist->data;
-			else		taglist->data = base->BWL2;
+			if (cfgure)	{
+				base->BWL2 = taglist->data;
+				config_set_int (SECTION, "BWL_Second", taglist->data);
+			} else
+				taglist->data = base->BWL2;
 			break;
 		case TAG_BWLLAST:
-			if (cfgure)	base->BWL3 = taglist->data;
-			else		taglist->data = base->BWL3;
+			if (cfgure)	{
+				base->BWL3 = taglist->data;
+				config_set_int (SECTION, "BWL_Last", taglist->data);
+			} else
+				taglist->data = base->BWL3;
 			break;
 		case TAG_CURRENTBWL:
 			if (cfgure)	tagerror (taglist, E_NOWRITE);
@@ -531,53 +486,73 @@ power_handle_tags (int cfgure, struct tagitem *taglist)
 			if (cfgure) {
 				base->flags.cpuload_enable = taglist->data;
 				base->flags.sleeplock_cpu = taglist->data;
+				config_set_boolean (SECTION, "CPULoad_sleeplock", taglist->data);
 			} else
 				taglist->data = base->flags.cpuload_enable;
 			break;
 		case TAG_CPULOADMIN:
-			if (cfgure)	base->cpuload_min = taglist->data;
-			else		taglist->data = base->cpuload_min;
+			if (cfgure)	{
+				base->cpuload_min = taglist->data;
+				config_set_percent (SECTION, "CPULoad_min", taglist->data);
+			} else
+				taglist->data = base->cpuload_min;
 			break;
 		case TAG_CPULOADPERIOD:
-			if (cfgure)	base->cpuload_period = taglist->data;
-			else		taglist->data = base->cpuload_period;
+			if (cfgure)	{
+				base->cpuload_period = taglist->data;
+				config_set_int (SECTION, "CPULoad_period", taglist->data);
+			} else
+				taglist->data = base->cpuload_period;
 			break;
 		case TAG_NETLOADSLEEPLOCK:
 			if (cfgure) {
 				base->flags.netload_enable = taglist->data;
 				base->flags.sleeplock_net = taglist->data;
+				config_set_boolean (SECTION, "NETLoad_sleeplock", taglist->data);
 			} else
 				taglist->data = base->flags.netload_enable;
 			break;
 		case TAG_NETLOADMIN:
-			if (cfgure)	base->netload_min = taglist->data;
-			else		taglist->data = base->netload_min;
+			if (cfgure)	{
+				base->netload_min = taglist->data;
+				config_set_int (SECTION, "NETLoad_min", taglist->data);
+			} else
+				taglist->data = base->netload_min;
 			break;
 		case TAG_NETLOADPERIOD:
-			if (cfgure)	base->netload_period = taglist->data;
-			else		taglist->data = base->netload_period;
+			if (cfgure)	{
+				base->netload_period = taglist->data;
+				config_set_int (SECTION, "NetLoad_period", taglist->data);
+			} else
+				taglist->data = base->netload_period;
 			break;
 		case TAG_NETLOADDEV:
-			if (cfgure)	strncpy (base->netload_dev, (char *) taglist->data, STDBUFFERLEN-1);
-			else		taglist->data = (long) base->netload_dev;
+			if (cfgure)	{
+				replace_string (&base->netload_dev, (char *) taglist->data);
+				config_set_string (SECTION, "NETLoad_device", base->netload_dev);
+			} else
+				taglist->data = (long) base->netload_dev;
 			break;
 		case TAG_EMERGENCYACTION:
 			if (cfgure) {
-				if (taglist->data > EMA_LAST)
-					tagerror(taglist, E_INVALID);
-				else if (taglist->data == EMA_SLEEP) {
-					if (base->flags.sleep_supported) /* sleep supported? */
-						base->emergency = EMA_SLEEP; /* then set to EMA_SLEEP */
-					else
+				if (taglist->data <= EMA_LAST) {
+					if (taglist->data == EMA_SLEEP && !base->flags.sleep_supported) {
 						tagerror (taglist, E_NOSUPPORT); /* otherwise send error */
-				} else
+						break;
+					}
 					base->emergency = taglist->data;
+					config_set_option (SECTION, "EmergencyAction", EmergencyList, taglist->data);
+				} else
+					tagerror(taglist, E_INVALID);
 			} else
 				taglist->data = (long) base->emergency;
 			break;
 		case TAG_HEARTBEATBEEP:
-			if (cfgure)	base->flags.heartbeat_enable = taglist->data;
-			else		taglist->data = base->flags.heartbeat_enable;
+			if (cfgure)	{
+				base->flags.heartbeat_enable = taglist->data;
+				config_set_boolean (SECTION, "HeartbeatBeep", taglist->data);
+			} else
+				taglist->data = base->flags.heartbeat_enable;
 			break;
 		case TAG_POWERCHANGED:  /* private tag */
 			if (cfgure) {
@@ -812,13 +787,15 @@ power_check_battery (int time, int psource)
 {
 	struct moddata_power *base = &modbase_power;
 	struct tagitem taglist[3];
+	int warnlevel[10], n, wl;
 
-	int warnlevel[] = {1200, base->BWL1 + BAT_HYSTERESIS,
-	                       base->BWL1, base->BWL2 + BAT_HYSTERESIS,
-	                       base->BWL2, base->BWL3 + BAT_HYSTERESIS,
-	                       base->BWL3, 1,
-	                       1,          0};
-	int n, wl;
+	warnlevel[0] = 1200;
+	warnlevel[1] = base->BWL1 + BAT_HYSTERESIS; warnlevel[2] = base->BWL1;
+	warnlevel[3] = base->BWL2 + BAT_HYSTERESIS; warnlevel[4] = base->BWL2;
+	warnlevel[5] = base->BWL3 + BAT_HYSTERESIS; warnlevel[6] = base->BWL3;
+    warnlevel[7] = 1;
+	warnlevel[8] = 1;
+	warnlevel[9] = 0;
 
 	/* if pbbuttons was started with AC plugged in, timeleft would
 	 * remain at -1 and after AC pulled out the value wasn't changed
@@ -962,7 +939,6 @@ void
 power_setpolicy (int policy, int powersource)
 {
 	struct moddata_power *base = &modbase_power;
-	char *policies[] = {POLICY_NONE_NAME, POLICY_POWERSAVE_NAME, POLICY_USER_NAME, POLICY_PERFORMANCE_NAME};
 	char *powersrc[] = {"battery", "ac"};
 	int change = 0;
 
@@ -975,7 +951,7 @@ power_setpolicy (int policy, int powersource)
 		change = 1;
 	}
 	if (change)
-		call_script (base->script_pmcs, policies[base->policy], powersrc[base->powersource], "");
+		call_script (base->script_pmcs, PolicyList[base->policy], powersrc[base->powersource], "");
 }
 
 int
