@@ -18,15 +18,12 @@
 
 /* public prototypes */
 int system_init ();
-int system_open (struct tagitem *taglist);
-int system_close ();
-int system_exit ();
+void system_exit ();
 
 /* private prototypes */
 void system_configure (struct tagitem *taglist);
 void system_query (struct tagitem *taglist);
 void system_handle_tags (int cfgure, struct tagitem *taglist);
 void system_timer1000 (struct tagitem *taglist);
-void system_config_writable (char *configreadfile, char *configwritefile);
 
 #endif    /* INCLUDE_MODULE_SYSTEM_H */
