@@ -1,15 +1,16 @@
-/* ----------------------------------------------------------------------------
- * pbbuttons.c
- * Daemon to put the Apple Powerbook special keys into action.
- * This version works as deamon without any interface.
+/**
+ * @brief Pbbuttonsd main program file
  *
- * Copyright 2002 Matthias Grimm
+ * PBButtonsd is a system deamon for mobile computers.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version
- * 2 of the License, or (at your option) any later version.
- * ----------------------------------------------------------------------------*/
+ * version 2 as published by the Free Software Foundation
+ * (http://www.gnu.org/licenses/gpl.html)
+ *
+ * @file    src/pbbuttonsd.c
+ * @author  Matthias Grimm <matthias.grimm@users.sourceforge.net>
+ */
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
@@ -30,6 +31,8 @@
 #include "input_manager.h"
 #include "gettext_macros.h"
 
+#include "class_config.h"
+
 int main(int argc, char *argv[])
 {
 	struct serverdata sdata, *sd = &sdata;
@@ -45,6 +48,7 @@ int main(int argc, char *argv[])
 	init_libpbb();              /* initialising pbblib */
 	init_serverdata (sd);        /* fill serverdata structure with default values */
 	if ((rc = evaluate_args(sd, argc, argv)) == 0) {  /* evaluate command line */
+		config_new (sd->configfile);
 		if ((rc = init_modules (sd)) == 0) {      /* call known modules for initialisation */
 			tval = process_queue_single (QUERYQUEUE, TAG_IDENTITY, 0);
 			if (!sd->quiet && tval)
@@ -68,6 +72,7 @@ int main(int argc, char *argv[])
 				g_main_loop_unref (sd->mainloop);
 			}
 			exit_modules (sd);  /* let modules cleanup their stuff */
+			config_destroy ();
 			rc = 0;
 		}
 	}
