@@ -35,65 +35,68 @@
  */
 extern pid_t getpgid (pid_t pid);
 
-int
-copy_path (char *source, char *dest, int type, int flags)
+gboolean
+check_path (char *source, int type, int flags)
 {
 	char *whitespace = NULL;
 	int rc = 0, perms = 0600;
 
 	if (type == TYPE_DIR) perms = 0700;
 
-	if (strlen (source) < STDBUFFERLEN) {
-		if ((whitespace = strchr (source, ' ')) != NULL)
-			*whitespace = 0;
+	if ((whitespace = strchr (source, ' ')) != NULL)
+		*whitespace = 0;
 
-		rc = check_devorfile (source, type);
-		if ((rc == 0) && (flags & CPFLG_PBBONLY))
-			rc = check_permissions (source, geteuid(), perms, 022);
-		if ((rc == E_NOEXIST) && (flags & CPFLG_MAYBEMISSED))
-			rc = 0;
-		if (type == TYPE_FILE && strlen(source) == 0)
-			rc = 0;  /* empty _file_names are acceptable */
+	rc = check_devorfile (source, type);
+	if ((rc == 0) && (flags & CPFLG_PBBONLY))
+		rc = check_permissions (source, geteuid(), perms, 022);
+	if ((rc == E_NOEXIST) && (flags & CPFLG_MAYBEMISSED))
+		rc = 0;
+	if (type == TYPE_FILE && strlen(source) == 0)
+		rc = 0;  /* empty _file_names are acceptable */
 
-		switch (rc) {
-		case E_NOEXIST:
-			print_msg (PBB_ERR, _("The object '%s' doesn't exist.\n"), source);
-			break;
-		case E_NOFILE:
-			print_msg (PBB_ERR, _("The object '%s' is not a file.\n"), source);
-			break;
-		case E_NODIR:
-			print_msg (PBB_ERR, _("The object '%s' is not a directory.\n"), source);
-			break;
-		case E_NOCHAR:
-			print_msg (PBB_ERR, _("The object '%s' is not a character device.\n"), source);
-			break;
-		case E_NOBLK:
-			print_msg (PBB_ERR, _("The object '%s' is not a block device.\n"), source);
-			break;
-		case E_USER:
-			print_msg (PBB_WARN, _("SECURITY: %s must be owned by the same owner as pbbuttonsd.\n"), source);
-			break;
-		case E_RIGHTS:
-			print_msg (PBB_WARN, _("SECURITY: %s must only be writable by the owner of pbbuttonsd.\n"), source);
-			break;
-		case E_PERM:
-			print_msg (PBB_ERR, _("Insufficient permissions for object %s, at least '0%o' needed.\n"), source, perms);
-			break;
-		default:
-			if (whitespace != NULL) *whitespace = ' ';
-			if (dest) strncpy(dest, source, STDBUFFERLEN);
-			break;
-		}
-	} else {
-		print_msg (PBB_ERR, _("Path of '%s' is to long.\n"), source);
-		rc = E_BUFOVL;
+	switch (rc) {
+	case E_NOEXIST:
+		print_msg (PBB_ERR, _("The object '%s' doesn't exist.\n"), source);
+		break;
+	case E_NOFILE:
+		print_msg (PBB_ERR, _("The object '%s' is not a file.\n"), source);
+		break;
+	case E_NODIR:
+		print_msg (PBB_ERR, _("The object '%s' is not a directory.\n"), source);
+		break;
+	case E_NOCHAR:
+		print_msg (PBB_ERR, _("The object '%s' is not a character device.\n"), source);
+		break;
+	case E_NOBLK:
+		print_msg (PBB_ERR, _("The object '%s' is not a block device.\n"), source);
+		break;
+	case E_USER:
+		print_msg (PBB_WARN, _("SECURITY: %s must be owned by the same owner as pbbuttonsd.\n"), source);
+		break;
+	case E_RIGHTS:
+		print_msg (PBB_WARN, _("SECURITY: %s must only be writable by the owner of pbbuttonsd.\n"), source);
+		break;
+	case E_PERM:
+		print_msg (PBB_ERR, _("Insufficient permissions for object %s, at least '0%o' needed.\n"), source, perms);
+		break;
 	}
 
 	if (whitespace != NULL) *whitespace = ' ';
 	return rc;
 }
 
+void
+replace_string (char **oldstr, char *newstr)
+{
+	GString *tmp;
+
+	if (*oldstr) g_free (*oldstr);
+
+	tmp = g_string_new (newstr);
+	*oldstr = tmp->str;
+	g_string_free (tmp, FALSE);
+}
+
 #if 0
 
 /* lauching of external programs with glib functions. The program will be
@@ -182,7 +185,7 @@ cbChildExit (GPid pin, gint status, gpointer data)
 	g_free (cmd);
 }
 
-//struct ChildData*
+/* struct ChildData* */
 int
 call_script (char* template, ...)
 {
@@ -215,7 +218,7 @@ call_script (char* template, ...)
 			g_atomic_int_inc (&cd->ref);
 			cd->cwatch = g_child_watch_add_full (G_PRIORITY_DEFAULT,
 					cd->pid, cbChildExit, (gpointer) cd, cbChildCleanup);
-//			return cd;
+/*			return cd; */
 			return rc;
 		} else {
 			print_msg (PBB_INFO, _("Script '%s' can't be launched - fork() failed.\n"), cmd);
@@ -228,7 +231,7 @@ call_script (char* template, ...)
 	}
 	g_strfreev (cd->argv);
 	g_free (cd);
-//	return NULL;
+/*	return NULL; */
 	return rc;
 }
 #endif
