@@ -26,7 +26,8 @@ struct ChildData {
 };
 
 /* prototypes */
-int copy_path (char *source, char *dest, int type, int flags);
+gboolean check_path (char *source, int type, int flags);
+void replace_string (char **oldstr, char *newstr);
 int call_script (char* template, ...);
 
 #endif    /* INCLUDE_SUPPORT_H */
