@@ -18,6 +18,13 @@
 
 #ifdef SYS_POWERBOOK
 #  define WITH_MODULE_PMAC
+#  define WITH_BACKLIGHT_PMU
+#endif
+
+#ifdef SYS_MACBOOK
+#  define WITH_MODULE_ACPI
+#  define WITH_MODULE_IMAC
+#  define WITH_BACKLIGHT_PCI
 #endif
 
 #ifdef SYS_I386_ACPI
